/**
 * @name pdfemb_trans
 * @type {object}
 */

import * as pdfjsLib from 'pdfjs-dist';
import { UNSUPPORTED_FEATURES } from 'pdfjs-dist';
import { DEFAULT_SCALE_VALUE, getActiveOrFocusedElement, isValidRotation, isValidScrollMode, isValidSpreadMode, normalizeWheelEventDirection, parseQueryString, ProgressBar, RenderingStates, ScrollMode } from './ui_utils.js';
import { PDFFindController } from './pdf_find_controller.js';
import { PDFFindBar } from './pdf_find_bar.js';
import { OverlayManager } from './overlay_manager.js';
import { AppOptions } from './app_options.js';
import { CursorTool, PDFCursorTools } from './pdf_cursor_tools.js';
import * as pdfjsViewer from 'pdfjs-dist/web/pdf_viewer.js';
import { ExternalServices } from './external_services.js';
import { PDFPresentationMode } from './pdf_presentation_mode.js';
import { Toolbar } from './toolbar.js';

const DISABLE_AUTO_FETCH_LOADING_BAR_TIMEOUT = 5000; // ms
const FORCE_PAGES_LOADED_TIMEOUT = 10000; // ms
const WHEEL_ZOOM_DISABLED_TIMEOUT = 1000; // ms

const USE_ONLY_CSS_ZOOM = true;
const DEFAULT_SCALE_DELTA = 1.1;
const MIN_SCALE = 0.25;
const MAX_SCALE = 10.0;

pdfjsLib.GlobalWorkerOptions.workerSrc = pdfemb_trans.workerSrc;

const PDFViewerApplication = {
	initialBookmark: document.location.hash.substring(1),
	appConfig: null,
	pdfLoadingTask: null,
	pdfDocument: null,
	pdfViewer: null,
	pdfHistory: null,
	pdfLinkService: null,
	eventBus: null,
	downloadManager: null,
	store: null,
	l10n: null,
	/** @type {Toolbar} */
	toolbar: null,
	_boundEvents: Object.create(null),
	/** @type {SecondaryToolbar} */
	secondaryToolbar: null,
	url: '',
	baseUrl: '',
	_downloadUrl: '',
	externalServices: ExternalServices,
	isInitialViewSet: false,
	downloadComplete: false,
	fullscreenMode: pdfemb_trans.fullscreenMode,
	watermarkPagesRendered: [],

	// Called once when the document is loaded.
	async initalize(appConfig) {
		this.appConfig = appConfig;
		await this.intializeWebView();
		await this._initializeL10n();

		this.bindEvents();
		this.bindWindowEvents();

		// We can start UI localization now.
		let appContainer = appConfig.appContainer || document.documentElement;
		this.l10n.translate(appContainer).then(() => {
			// Dispatch the 'localized' event on the `eventBus` once the viewer
			// has been fully initialized and translated.
			this.eventBus.dispatch( 'localized' );
		});
	},
	/**
	 * @private
	 */
	async _initializeL10n() {
		this.l10n = this.externalServices.createL10n( { locale: AppOptions.get("locale") } );
		document.getElementsByTagName("html")[0].dir = await this.l10n.getDirection();
	},
	async intializeWebView() {
		const { appConfig } = this;
		let self = this;
		const eventBus = new pdfjsViewer.EventBus();
		this.eventBus = eventBus;

		this.overlayManager = new OverlayManager();

		// Set how the links inside the PDF files will be opened.
		AppOptions.set( 'externalLinkTarget', pdfemb_trans.newwindow === 'on' ? 2 : 3 );
		AppOptions.set( 'verbosity', 0 );
		AppOptions.set( 'isEvalSupported', false );

		const linkService = new pdfjsViewer.PDFLinkService({
			eventBus,
			externalLinkTarget: AppOptions.get('externalLinkTarget'),
			externalLinkRel: AppOptions.get('externalLinkRel'),
			ignoreDestinationZoom: AppOptions.get('ignoreDestinationZoom'),
		});
		this.pdfLinkService = linkService;

		const findController = new PDFFindController({
			linkService: linkService,
			eventBus,
		});
		this.findController = findController;
		this.l10n = pdfjsViewer.NullL10n;

		// const container = document.getElementById('viewerContainer');
		const container = appConfig.mainContainer;
		const pdfViewer = new pdfjsViewer.PDFViewer({
			container,
			eventBus,
			linkService,
			findController,
			l10n: this.l10n,
			useOnlyCssZoom: USE_ONLY_CSS_ZOOM,
			textLayerMode: AppOptions.get('textLayerMode'),
			_currentScaleValue: 'page-fit',
		});

		this.pdfViewer = pdfViewer;

		linkService.setViewer(pdfViewer);

		this.pdfHistory = new pdfjsViewer.PDFHistory({
			eventBus,
			linkService,
		});
		linkService.setHistory(this.pdfHistory);

		if (!this.supportsIntegratedFind) {
			this.findBar = new PDFFindBar(
				appConfig.findBar,
				eventBus,
				this.l10n,
			);
		}

		this.pdfCursorTools = new PDFCursorTools({
			container,
			eventBus,
			cursorToolOnLoad: CursorTool.HAND,
		});

		this.toolbar = new Toolbar( appConfig.toolbar, eventBus, this.l10n );

		if ( this.supportsFullscreen ) {
			this.pdfPresentationMode = new PDFPresentationMode( {
				container,
				pdfViewer: this.pdfViewer,
				eventBus,
			} );
		}

		/**
		 * Mobile fullscreen functionality.
		 */
		// Always hide the "view in full screen" mobile-only text in fullscreen.
		if ( this.fullscreenMode === 'on' ) {
			$( '.pdfemb-inner-div-wantmobile-fswrap' ).hide();
		}

		// Open fullscreen when clicked.
		$( '.pdfemb-wantmobile-fsarea' ).on( 'click', function( event ) {
			$( event.target ).parent().hide();
			// The top toolbar is always present on a page even though it may be hidden.
			document.getElementsByClassName( 'wppdf-fullscreen-button' )[ 0 ].click();
			window.parent.document.body.style.overflow = 'hidden';
		} );

		// Track view/download file actions.
		eventBus.on( 'pagesloaded', function () {
			pdfembTrackPDF( 'view' );
		} );

		// Set the user settings.
		eventBus.on( 'pagesinit', function() {
			let data = document.getElementById( 'pdf-data' );
			let pageNum = data.getAttribute( 'data-page' );
			let scale = data.getAttribute( 'data-scale' );
			let toolbarLocation = data.getAttribute( 'data-toolbar' );

			let realScale = '';

			if ( pageNum !== null ) {
				self.pdfLinkService.goToPage( Math.max( 1, parseInt( pageNum, 10 ) ) );
			}

			// Set the PDF SCALE (Zoom).
			let zoomValue = isNaN( pdfemb_trans.zoom ) ? pdfemb_trans.zoom : parseInt( pdfemb_trans.zoom, 10 ) / 100;
			self.pdfViewer.currentScaleValue = zoomValue;
			for ( let i = 0; i < self.appConfig.toolbar.scaleSelect.length; i++ ) {
				self.appConfig.toolbar.scaleSelect[i].value = zoomValue;
			}

			self.pdfViewer.scrollMode = pdfemb_trans.continousscroll ? ScrollMode.VERTICAL : ScrollMode.PAGE;

			// Number of pages.
			if ( toolbarLocation === 'top' || toolbarLocation === 'both' ) {
				document.getElementById( 'numPages' ).innerHTML = self.pagesCount;
			}
			if ( toolbarLocation === 'bottom' || toolbarLocation === 'both' ) {
				document.getElementById( 'numPages-bottom' ).innerHTML = self.pagesCount;
			}

			pdfembEvaluateWantMobile();
		} );
		eventBus.on( 'pagechanging', function (evt) {
				let data = document.getElementById('pdf-data');
				let toolbarLocation = data.getAttribute('data-toolbar');

				if ( toolbarLocation === 'top' || toolbarLocation === 'both' ) {
					document.getElementById('pageNumber').value = evt.pageNumber;
					document.getElementById('numPages').innerHTML = self.pagesCount;
					document.getElementById('previous').disabled = evt.pageNumber <= 1;
					document.getElementById('next').disabled = evt.pageNumber >= self.pagesCount;
				}
				if ( toolbarLocation === 'bottom' || toolbarLocation === 'both' ) {
					document.getElementById('pageNumber-bottom').value = evt.pageNumber;
					document.getElementById('numPages-bottom').innerHTML = self.pagesCount;
					document.getElementById('previous-bottom').disabled = evt.pageNumber <= 1;
					document.getElementById('next-bottom').disabled = evt.pageNumber >= self.pagesCount;
				}
			},
			true,
		);
	},
	get supportsFullscreen() {
		return pdfjsLib.shadow(this, "supportsFullscreen", document.fullscreenEnabled);
	},
	/**
	 * @private
	 */
	async _initializeMetadata(pdfDocument) {
		const { info, metadata, contentDispositionFilename, contentLength } =
			await pdfDocument.getMetadata();

		if (pdfDocument !== this.pdfDocument) {
			return; // The document was closed while the metadata resolved.
		}
		this.documentInfo = info;
		this.metadata = metadata;
		this._contentDispositionFilename ??= contentDispositionFilename;
		this._contentLength ??= contentLength; // See `getDownloadInfo`-call above.

		let pdfTitle = info.Title;

		const metadataTitle = metadata?.get('dc:title');
		if (metadataTitle) {
			// Ghostscript can produce invalid 'dc:title' Metadata entries:
			//  - The title may be "Untitled" (fixes bug 1031612).
			//  - The title may contain incorrectly encoded characters, which thus
			//    looks broken, hence we ignore the Metadata entry when it contains
			//    characters from the Specials Unicode block (fixes bug 1605526).
			if (
				metadataTitle !== 'Untitled' &&
				!/[\uFFF0-\uFFFF]/g.test(metadataTitle)
			) {
				pdfTitle = metadataTitle;
			}
		}
		if (pdfTitle) {
			this.setTitle(
				`${pdfTitle} - ${
					this._contentDispositionFilename || this._title
				}`,
			);
		} else if (this._contentDispositionFilename) {
			this.setTitle(this._contentDispositionFilename);
		}

		if (
			info.IsXFAPresent &&
			!info.IsAcroFormPresent &&
			!pdfDocument.isPureXfa
		) {
			if (pdfDocument.loadingParams.enableXfa) {
				console.warn(
					'Warning: XFA Foreground documents are not supported',
				);
			} else {
				console.warn('Warning: XFA support is not enabled');
			}
			this.fallback(UNSUPPORTED_FEATURES.forms);
		} else if (
			(info.IsAcroFormPresent || info.IsXFAPresent) &&
			!this.pdfViewer.renderForms
		) {
			console.warn('Warning: Interactive form support is not enabled');
			this.fallback(UNSUPPORTED_FEATURES.forms);
		}

		if (info.IsSignaturesPresent) {
			console.warn(
				'Warning: Digital signatures validation is not supported',
			);
			this.fallback(UNSUPPORTED_FEATURES.signatures);
		}

		this.eventBus.dispatch('metadataloaded', { source: this });
	},

	async _initializePageLabels(pdfDocument) {
		const labels = await pdfDocument.getPageLabels();

		if (pdfDocument !== this.pdfDocument) {
			return; // The document was closed while the page labels resolved.
		}
		if (!labels || AppOptions.get('disablePageLabels')) {
			return;
		}
		const numLabels = labels.length;
		// Ignore page labels that correspond to standard page numbering, or page labels that are all empty.
		let standardLabels = 0,
			emptyLabels = 0;
		for (let i = 0; i < numLabels; i++) {
			const label = labels[i];
			if (label === (i + 1).toString()) {
				standardLabels++;
			} else if (label === '') {
				emptyLabels++;
			} else {
				break;
			}
		}
		if (standardLabels >= numLabels || emptyLabels >= numLabels) {
			return;
		}
		const { pdfViewer, pdfThumbnailViewer, toolbar } = this;

		pdfViewer.setPageLabels(labels);
		pdfThumbnailViewer?.setPageLabels(labels);

		// Changing toolbar page display to use labels, and we need to set the label of the current page.
		toolbar.setPagesCount(numLabels, true);
		toolbar.setPageNumber(
			pdfViewer.currentPageNumber,
			pdfViewer.currentPageLabel,
		);
	},

	run( config ) {
		this.initalize( config ).then( WebViewInitialized );
	},

	/**
	 * Opens PDF document specified by URL.
	 *
	 * @returns {Promise} - Returns the promise, which is resolved when document is opened.
	 */
	async open( file, args ) {
		const self = this;

		if ( this.pdfLoadingTask ) {
			// We need to destroy already opened document.
			await this.close();
		}

		const parameters = Object.create(null);

		if (typeof file === 'string') {
			// This is a regular URL for a not secured file.
			this.setTitleUsingUrl(file, /* downloadUrl = */ file);
			parameters.url = file;
		} else if (file && 'byteLength' in file) {
			// ArrayBuffer.
			parameters.data = file;
		} else if (file.url && file.originalUrl) {
			this.setTitleUsingUrl(
				file.originalUrl,
				/* downloadUrl = */ file.url,
			);
			parameters.url = file.url;
		}

		// Finally, update the API parameters with the arguments (if they exist).
		if ( args ) {
			for ( const key in args ) {
				parameters[ key ] = args[ key ];
			}
		}

		if ( parameters.secure && parameters.nonced_url ) {
			this._downloadUrl = parameters.nonced_url + '&d=1';
		} else {
			this._downloadUrl = parameters.url;
		}

		this._docFilename = parameters.file;
		parameters.cMapPacked = true;
		parameters.cMapUrl = pdfemb_trans.cMapUrl;

		// Do not allow scripts execution in FontMatrix used for rendering glyphs.
		parameters.isEvalSupported = AppOptions.get( 'isEvalSupported' );

		// Hiding messages like "TT: undefined function" which are bad font recovery messages.
		parameters.verbosity = AppOptions.get( 'verbosity' );

		// Loading document.
		const loadingTask = pdfjsLib.getDocument( parameters );

		this.pdfLoadingTask = loadingTask;
		loadingTask.onPassword = (updateCallback, reason) => {
			this.pdfLinkService.externalLinkEnabled = false;
			this.passwordPrompt.setUpdateCallback(updateCallback, reason);
			this.passwordPrompt.open();
		};
		loadingTask.onProgress = function (progressData) {
			self.progress(progressData.loaded / progressData.total);
		};

		return loadingTask.promise.then(
			// Success (onFulfilled).
			function (pdfDocument) {
				// Document loaded, specifying document for the viewer.
				self.pdfDocument = pdfDocument;
				self.pdfViewer.setDocument(pdfDocument);
				self.pdfLinkService.setDocument(pdfDocument);
				self.pdfHistory.initialize({
					fingerprint: pdfDocument.fingerprints[0],
				});
				webViewerZoomReset();
				self.loadingBar.hide();
				self.setTitleUsingMetadata(pdfDocument);
				self._initializePageLabels(pdfDocument);
			},
			// Fail (onRejected).
			function (exception) {
				const message = exception && exception.message;
				const l10n = self.l10n;
				let loadingErrorMessage;

				if (exception instanceof pdfjsLib.InvalidPDFException) {
					// change error message also for other builds
					loadingErrorMessage = l10n.get( 'invalid_file_error', null, 'Invalid or corrupted PDF file.' );
				} else if (exception instanceof pdfjsLib.MissingPDFException) {
					// special message for missing PDFs
					loadingErrorMessage = l10n.get( 'missing_file_error', null, 'Missing PDF file.' );
				} else if (
					exception instanceof pdfjsLib.UnexpectedResponseException
				) {
					loadingErrorMessage = l10n.get( 'unexpected_response_error', null, 'Unexpected server response.' );
				} else {
					loadingErrorMessage = l10n.get( 'loading_error', null, 'An error occurred while loading the PDF.' );
				}

				loadingErrorMessage.then(function (msg) {
					self.error(msg, { message });
				});

				self.loadingBar.hide();
			},
		);
	},

	/**
	 * Closes opened PDF document.
	 * @returns {Promise} - Returns the promise, which is resolved when all
	 *                      destruction is completed.
	 */
	close() {
		const errorWrapper = document.getElementById('errorWrapper');
		errorWrapper.hidden = true;

		if (!this.pdfLoadingTask) {
			return Promise.resolve();
		}

		const promise = this.pdfLoadingTask.destroy();
		this.pdfLoadingTask = null;

		if (this.pdfDocument) {
			this.pdfDocument = null;

			this.pdfViewer.setDocument(null);
			this.pdfLinkService.setDocument(null, null);

			if (this.pdfHistory) {
				this.pdfHistory.reset();
			}
		}

		return promise;
	},

	get loadingBar() {
		const bar = new ProgressBar('loadingBar');

		return pdfjsLib.shadow(this, 'loadingBar', bar);
	},

	setTitleUsingUrl: function pdfViewSetTitleUsingUrl(url) {
		this.url = url;
		let title = pdfjsLib.getFilenameFromUrl(url) || url;
		try {
			title = decodeURIComponent(title);
		} catch (e) {
			// decodeURIComponent may throw URIError,
			// fall back to using the unprocessed url in that case
		}

		this.setTitle(title);
	},

	setTitleUsingMetadata(pdfDocument) {
		const self = this;
		pdfDocument.getMetadata().then(function (data) {
			const info = data.info,
				metadata = data.metadata;
			self.documentInfo = info;
			self.metadata = metadata;

			let pdfTitle;
			if (metadata && metadata.has('dc:title')) {
				const title = metadata.get('dc:title');
				// Ghostscript sometimes returns 'Untitled', so prevent setting the title to 'Untitled'.
				if (title !== 'Untitled') {
					pdfTitle = title;
				}
			}

			if (!pdfTitle && info && info.Title) {
				pdfTitle = info.Title;
			}

			if (pdfTitle) {
				self.setTitle(pdfTitle + ' - ' + document.title);
			}
		});
	},

	setTitle: function pdfViewSetTitle(title) {
		document.title = title;
	},

	error: function pdfViewError(message, moreInfo) {
		const l10n = this.l10n;
		const moreInfoText = [];

		if (moreInfo) {
			moreInfoText.push(
				l10n.get(
					'error_message',
					{ message: moreInfo.message },
					'Error: {{message}}',
				),
			);
			if (moreInfo.stack) {
				moreInfoText.push(
					l10n.get(
						'error_stack',
						{ stack: moreInfo.stack },
						'Stack: {{stack}}',
					),
				);
			} else {
				if (moreInfo.filename) {
					moreInfoText.push(
						l10n.get(
							'error_file',
							{ file: moreInfo.filename },
							'File: {{file}}',
						),
					);
				}
				if (moreInfo.lineNumber) {
					moreInfoText.push(
						l10n.get(
							'error_line',
							{ line: moreInfo.lineNumber },
							'Line: {{line}}',
						),
					);
				}
			}
		}

		const errorWrapper = document.getElementById('errorWrapper');
		errorWrapper.hidden = false;

		const errorMessage = document.getElementById('errorMessage');
		errorMessage.textContent = message;

		const closeButton = document.getElementById('errorClose');
		closeButton.onclick = function () {
			errorWrapper.hidden = true;
		};

		const errorMoreInfo = document.getElementById('errorMoreInfo');
		const moreInfoButton = document.getElementById('errorShowMore');
		const lessInfoButton = document.getElementById('errorShowLess');
		moreInfoButton.onclick = function () {
			errorMoreInfo.hidden = false;
			moreInfoButton.hidden = true;
			lessInfoButton.hidden = false;
			errorMoreInfo.style.height = errorMoreInfo.scrollHeight + 'px';
		};
		lessInfoButton.onclick = function () {
			errorMoreInfo.hidden = true;
			moreInfoButton.hidden = false;
			lessInfoButton.hidden = true;
		};
		moreInfoButton.hidden = false;
		lessInfoButton.hidden = true;
		Promise.all(moreInfoText).then(function (parts) {
			errorMoreInfo.value = parts.join('\n');
		});
	},
	progress: function pdfViewProgress(level) {
		const percent = Math.round(level * 100);
		// Updating the bar if value increases.
		if (percent > this.loadingBar.percent || isNaN(percent)) {
			this.loadingBar.percent = percent;
		}
	},

	bindEvents() {
		const { eventBus, _boundEvents } = this;

		_boundEvents.beforePrint = this.beforePrint.bind( this );
		_boundEvents.afterPrint = this.afterPrint.bind( this );

		eventBus._on( 'resize', webViewerResize );
		eventBus._on( 'hashchange', webViewerHashchange );
		eventBus._on( 'beforeprint', _boundEvents.beforePrint );
		eventBus._on( 'afterprint', _boundEvents.afterPrint );
		eventBus._on( 'pagerendered', webViewerPageRendered );
		eventBus._on( 'updateviewarea', webViewerUpdateViewarea );
		eventBus._on( 'pagechanging', webViewerPageChanging );
		eventBus._on( 'scalechanging', webViewerScaleChanging );
		eventBus._on( 'rotationchanging', webViewerRotationChanging );
		eventBus._on( 'namedaction', webViewerNamedAction );
		eventBus._on( 'presentationmodechanged', webViewerPresentationModeChanged );
		eventBus._on( 'presentationmode', webViewerPresentationMode );
		eventBus._on( 'switchannotationeditormode', webViewerSwitchAnnotationEditorMode );
		eventBus._on( 'switchannotationeditorparams', webViewerSwitchAnnotationEditorParams );
		eventBus._on( 'print', webViewerPrint );
		eventBus._on( 'download', webViewerDownload );
		eventBus._on( 'fullscreen', webViewerFullscreen );
		eventBus._on( 'firstpage', webViewerFirstPage );
		eventBus._on( 'lastpage', webViewerLastPage );
		eventBus._on( 'nextpage', webViewerNextPage );
		eventBus._on( 'previouspage', webViewerPreviousPage );
		eventBus._on( 'zoomin', webViewerZoomIn );
		eventBus._on( 'zoomout', webViewerZoomOut );
		eventBus._on( 'zoomreset', webViewerZoomReset );
		eventBus._on( 'pagenumberchanged', webViewerPageNumberChanged );
		eventBus._on( 'scalechanged', webViewerScaleChanged );
		eventBus._on( 'rotatecw', webViewerRotateCw );
		eventBus._on( 'rotateccw', webViewerRotateCcw );
		eventBus._on( 'optionalcontentconfig', webViewerOptionalContentConfig );
		eventBus._on( 'switchscrollmode', webViewerSwitchScrollMode );
		eventBus._on( 'scrollmodechanged', webViewerScrollModeChanged );
		eventBus._on( 'spreadmodechanged', webViewerSpreadModeChanged );
		eventBus._on( 'documentproperties', webViewerDocumentProperties );
		eventBus._on( 'findfromurlhash', webViewerFindFromUrlHash );
		eventBus._on( 'updatefindmatchescount', webViewerUpdateFindMatchesCount );
		eventBus._on( 'updatefindcontrolstate', webViewerUpdateFindControlState );
	},
	bindWindowEvents() {
		const { eventBus, _boundEvents } = this;

		function addWindowResolutionChange( evt = null ) {
			if ( evt ) {
				webViewerResolutionChange( evt );
			}
			const mediaQueryList = window.matchMedia(
				`(resolution: ${window.devicePixelRatio || 1}dppx)`,
			);
			mediaQueryList.addEventListener( 'change', addWindowResolutionChange, { once: true } );

			if (
				typeof PDFJSDev !== 'undefined' &&
				PDFJSDev.test( 'MOZCENTRAL' )
			) {
				return;
			}
			_boundEvents.removeWindowResolutionChange ||= function() {
				mediaQueryList.removeEventListener( 'change', addWindowResolutionChange );
				_boundEvents.removeWindowResolutionChange = null;
			};
		}

		addWindowResolutionChange();

		_boundEvents.windowResize = () => {
			eventBus.dispatch( 'resize', { source: window } );
		};
		_boundEvents.windowHashChange = () => {
			eventBus.dispatch( 'hashchange', {
				source: window,
				hash: document.location.hash.substring( 1 ),
			} );
		};
		_boundEvents.windowBeforePrint = () => {
			eventBus.dispatch( 'beforeprint', { source: window } );
		};
		_boundEvents.windowAfterPrint = () => {
			eventBus.dispatch( 'afterprint', { source: window } );
		};
		_boundEvents.windowUpdateFromSandbox = ( event ) => {
			eventBus.dispatch( 'updatefromsandbox', {
				source: window,
				detail: event.detail,
			} );
		};

		window.addEventListener( 'visibilitychange', webViewerVisibilityChange );
		window.addEventListener( 'wheel', webViewerWheel, { passive: false } );
		window.addEventListener( 'touchmove', webViewerWheel, { passive: false } );
		window.addEventListener( 'touchstart', webViewerTouchStart, { passive: false } );
		window.addEventListener( 'keydown', webViewerKeyDown );
		window.addEventListener( 'resize', _boundEvents.windowResize );
		window.addEventListener( 'hashchange', _boundEvents.windowHashChange );
		window.addEventListener( 'beforeprint', _boundEvents.windowBeforePrint );
		window.addEventListener( 'afterprint', _boundEvents.windowAfterPrint );
		window.addEventListener( 'updatefromsandbox', _boundEvents.windowUpdateFromSandbox );
	},
	unbindEvents() {
		if ( typeof PDFJSDev !== 'undefined' && PDFJSDev.test( 'MOZCENTRAL' ) ) {
			throw new Error( 'Not implemented: unbindEvents' );
		}
		const { eventBus, _boundEvents } = this;

		eventBus._off( 'resize', webViewerResize );
		eventBus._off( 'hashchange', webViewerHashchange );
		eventBus._off( 'beforeprint', _boundEvents.beforePrint );
		eventBus._off( 'afterprint', _boundEvents.afterPrint );
		eventBus._off( 'pagerendered', webViewerPageRendered );
		eventBus._off( 'updateviewarea', webViewerUpdateViewarea );
		eventBus._off( 'pagechanging', webViewerPageChanging );
		eventBus._off( 'scalechanging', webViewerScaleChanging );
		eventBus._off( 'rotationchanging', webViewerRotationChanging );
		eventBus._off( 'namedaction', webViewerNamedAction );
		eventBus._off( 'presentationmodechanged', webViewerPresentationModeChanged );
		eventBus._off( 'presentationmode', webViewerPresentationMode );
		eventBus._off( 'print', webViewerPrint );
		eventBus._off( 'download', webViewerDownload );
		eventBus._off( 'fullscreen', webViewerFullscreen );
		eventBus._off( 'firstpage', webViewerFirstPage );
		eventBus._off( 'lastpage', webViewerLastPage );
		eventBus._off( 'nextpage', webViewerNextPage );
		eventBus._off( 'previouspage', webViewerPreviousPage );
		eventBus._off( 'zoomin', webViewerZoomIn );
		eventBus._off( 'zoomout', webViewerZoomOut );
		eventBus._off( 'zoomreset', webViewerZoomReset );
		eventBus._off( 'pagenumberchanged', webViewerPageNumberChanged );
		eventBus._off( 'scalechanged', webViewerScaleChanged );
		eventBus._off( 'rotatecw', webViewerRotateCw );
		eventBus._off( 'rotateccw', webViewerRotateCcw );
		eventBus._off( 'optionalcontentconfig', webViewerOptionalContentConfig );
		eventBus._off( 'switchscrollmode', webViewerSwitchScrollMode );
		eventBus._off( 'scrollmodechanged', webViewerScrollModeChanged );
		eventBus._off( 'switchspreadmode', webViewerSwitchSpreadMode );
		eventBus._off( 'spreadmodechanged', webViewerSpreadModeChanged );
		eventBus._off( 'documentproperties', webViewerDocumentProperties );
		eventBus._off( 'findfromurlhash', webViewerFindFromUrlHash );
		eventBus._off( 'updatefindmatchescount', webViewerUpdateFindMatchesCount );
		eventBus._off( 'updatefindcontrolstate', webViewerUpdateFindControlState );

		if ( _boundEvents.reportPageStatsPDFBug ) {
			eventBus._off( 'pagerendered', _boundEvents.reportPageStatsPDFBug );
			eventBus._off( 'pagechanging', _boundEvents.reportPageStatsPDFBug );

			_boundEvents.reportPageStatsPDFBug = null;
		}
		if ( typeof PDFJSDev === 'undefined' || PDFJSDev.test( 'GENERIC' ) ) {
			eventBus._off( 'fileinputchange', webViewerFileInputChange );
			eventBus._off( 'openfile', webViewerOpenFile );
		}

		_boundEvents.beforePrint = null;
		_boundEvents.afterPrint = null;
	},
	unbindWindowEvents() {
		if (typeof PDFJSDev !== 'undefined' && PDFJSDev.test('MOZCENTRAL')) {
			throw new Error('Not implemented: unbindWindowEvents');
		}
		const { _boundEvents } = this;

		window.removeEventListener( 'visibilitychange', webViewerVisibilityChange );
		window.removeEventListener('wheel', webViewerWheel, { passive: false });
		window.removeEventListener('touchstart', webViewerTouchStart, { passive: false });
		window.removeEventListener('keydown', webViewerKeyDown);
		window.removeEventListener('resize', _boundEvents.windowResize);
		window.removeEventListener('hashchange', _boundEvents.windowHashChange);
		window.removeEventListener( 'beforeprint', _boundEvents.windowBeforePrint );
		window.removeEventListener('afterprint', _boundEvents.windowAfterPrint);
		window.removeEventListener( 'updatefromsandbox', _boundEvents.windowUpdateFromSandbox );

		_boundEvents.removeWindowResolutionChange?.();
		_boundEvents.windowResize = null;
		_boundEvents.windowHashChange = null;
		_boundEvents.windowBeforePrint = null;
		_boundEvents.windowAfterPrint = null;
		_boundEvents.windowUpdateFromSandbox = null;
	},

	beforePrint() {
		this._printAnnotationStoragePromise = this.pdfScriptingManager
			.dispatchWillPrint()
			.catch(() => {
				/* Avoid breaking printing; ignoring errors. */
			})
			.then(() => {
				return this.pdfDocument?.annotationStorage.print;
			});

		if (this.printService) {
			// There is no way to suppress beforePrint/afterPrint events,
			// but PDFPrintService may generate double events -- this will ignore
			// the second event that will be coming from native window.print().
			return;
		}

		if (!this.supportsPrinting) {
			this.l10n.get('printing_not_supported').then((msg) => {
				this._otherError(msg);
			});
			return;
		}

		// The beforePrint is a sync method and we need to know layout before
		// returning from this method. Ensure that we can get sizes of the pages.
		if (!this.pdfViewer.pageViewsReady) {
			this.l10n.get('printing_not_ready').then((msg) => {
				// eslint-disable-next-line no-alert
				window.alert(msg);
			});
			return;
		}

		const pagesOverview = this.pdfViewer.getPagesOverview();
		const printContainer = this.appConfig.printContainer;
		const printResolution = AppOptions.get('printResolution');
		const optionalContentConfigPromise =
			this.pdfViewer.optionalContentConfigPromise;

		const printService = PDFPrintServiceFactory.instance.createPrintService(
			this.pdfDocument,
			pagesOverview,
			printContainer,
			printResolution,
			optionalContentConfigPromise,
			this._printAnnotationStoragePromise,
			this.l10n,
		);
		this.printService = printService;
		this.forceRendering();

		printService.layout();
	},

	_ensureDownloadComplete() {
		if (this.pdfDocument && this.downloadComplete) {
			return;
		}
		throw new Error('PDF document not downloaded.');
	},
	goFullscreen() {},
	triggerFullscreen( target ) {
		let pdf_target_id = target.getAttribute( 'data-pdf-id' );
		let frame = window.parent.document.querySelector( '#wppdf-emb-iframe-' + pdf_target_id );
		let container = window.parent.document.querySelector( '#wppdfemb-frame-container-' + pdf_target_id );

		let frameSrc = new URL( frame.getAttribute('src') );

		if ( frame.classList.contains( 'wppdf-fullscreen' ) ) {
			frameSrc.searchParams.delete( 'fullscreenMode' );
			frame.setAttribute( 'src', frameSrc.toString() );

			container.append( frame );
			frame.classList.remove( 'wppdf-fullscreen' );
		}
		else {
			frameSrc.searchParams.append('fullscreenMode', 'on');
			frame.setAttribute( 'src', frameSrc.toString() );

			frame.classList.add( 'wppdf-fullscreen' );
			parent.document.body.append( frame );
		}
	},
	isFullscreen() {
		return this.fullscreenMode === 'on';
	},
	resetFullscreen() {
		this.fullscreenMode = 'off';
	},

	async download() {
		const url = this._downloadUrl,
			filename = new URL( url ).pathname.split( '/' ).pop();

		let link = document.createElement( 'a' );

		link.setAttribute( 'href', url );
		link.setAttribute( 'download', filename );

		link.dispatchEvent( new MouseEvent( 'click' ) );

		pdfembTrackPDF( 'download' );
	},

	afterPrint() {
		if (this._printAnnotationStoragePromise) {
			this._printAnnotationStoragePromise.then(() => {
				this.pdfScriptingManager.dispatchDidPrint();
			});
			this._printAnnotationStoragePromise = null;
		}

		if (this.printService) {
			this.printService.destroy();
			this.printService = null;

			this.pdfDocument?.annotationStorage.resetModified();
		}
		this.forceRendering();
	},

	// Triggered only when pressing + and - buttons.
	zoomIn: function pdfViewZoomIn(ticks) {

		let newScale = this.pdfViewer.currentScale;
		do {
			newScale = (newScale * DEFAULT_SCALE_DELTA).toFixed(2);
			newScale = Math.ceil(newScale * 10) / 10;
			newScale = Math.min(MAX_SCALE, newScale);
		} while (--ticks && newScale < MAX_SCALE);

		this.pdfViewer.currentScaleValue = newScale;
	},

	// Triggered only when pressing + and - buttons.
	zoomOut: function pdfViewZoomOut(ticks) {
		let newScale = this.pdfViewer.currentScale;
		do {
			newScale = (newScale / DEFAULT_SCALE_DELTA).toFixed(2);
			newScale = Math.floor(newScale * 10) / 10;
			newScale = Math.max(MIN_SCALE, newScale);
		} while (--ticks && newScale > MIN_SCALE);
		this.pdfViewer.currentScaleValue = newScale;
	},

	// Triggered on load plus fullscreen mode init.
	zoomReset() {
		if (this.pdfViewer.isInPresentationMode) {
			return;
		}
		this.pdfViewer.currentScaleValue = DEFAULT_SCALE_VALUE;
	},
	requestPresentationMode() {
		this.pdfPresentationMode?.request();
	},

	get pagesCount() {
		return this.pdfDocument ? this.pdfDocument.numPages : 0;
	},

	get page() {
		return this.pdfViewer.currentPageNumber;
	},

	set page(val) {
		this.pdfViewer.currentPageNumber = val;
	},
};

function webViewerHashchange(evt) {
	const hash = evt.hash;
	if (!hash) {
		return;
	}
	if (!PDFViewerApplication.isInitialViewSet) {
		PDFViewerApplication.initialBookmark = hash;
	} else if (!PDFViewerApplication.pdfHistory?.popStateInProgress) {
		PDFViewerApplication.pdfLinkService.setHash(hash);
	}
}
function webViewerPageRendered( event ) {
	// If the page is still visible when it has finished rendering,
	// ensure that the page number input loading indicator is hidden.
	if ( PDFViewerApplication.page === event.pageNumber) {
		PDFViewerApplication.toolbar.updateLoadingIndicatorState(false);
	}

	if (event.error) {
		PDFViewerApplication.l10n.get('rendering_error').then((msg) => {
			PDFViewerApplication._otherError(msg, event.error);
		});
	}

	// Watermarks need to be rendered for current and previous pages - if they are not already rendered.
	// We cache the page processed not to re-render twice which doubles the opacity.
	let curPage = document.getElementById('viewer').querySelector( '[data-page-number="' + event.pageNumber + '"]' );

	if ( curPage && ! PDFViewerApplication.watermarkPagesRendered.includes( event.pageNumber ) ) {
		let ctx = curPage.querySelector( 'canvas' )?.getContext( '2d' );

		renderWatermark( ctx, event.pageNumber );

		PDFViewerApplication.watermarkPagesRendered.push( event.pageNumber );
	} else {
		let prevPage = document.getElementById('viewer').querySelector( '[data-page-number="' + ( event.pageNumber - 1 ) + '"]' );

		if ( prevPage && ! PDFViewerApplication.watermarkPagesRendered.includes( event.pageNumber - 1 ) ) {
			let ctx = prevPage.querySelector( 'canvas' )?.getContext( '2d' );

			renderWatermark( ctx, event.pageNumber - 1 );

			PDFViewerApplication.watermarkPagesRendered.push( event.pageNumber - 1 );
		} else {
			if ( ! PDFViewerApplication.watermarkPagesRendered.includes( event.pageNumber ) ) {
				let nextPage = document.getElementById('viewer').querySelector( '[data-page-number="' + ( event.pageNumber + 1 ) + '"]' );

				if ( nextPage ) {
					let ctx = nextPage.querySelector( 'canvas' )?.getContext( '2d' );

					renderWatermark( ctx, event.pageNumber + 1 );

					PDFViewerApplication.watermarkPagesRendered.push( event.pageNumber + 1 );
				}
			}
		}
	}
}
function webViewerUpdateViewarea({ location }) {
	if (PDFViewerApplication.isInitialViewSet) {
		// Only update the storage when the document has been loaded *and* rendered.
		PDFViewerApplication.store
			?.setMultiple({
				page: location.pageNumber,
				zoom: location.scale,
				scrollLeft: location.left,
				scrollTop: location.top,
				rotation: location.rotation,
			})
			.catch(() => {
				// Unable to write to storage.
			});
	}

	// Show/hide the loading indicator in the page number input element.
	const currentPage = PDFViewerApplication.pdfViewer.getPageView(
		/* index = */ PDFViewerApplication.page - 1,
	);
	const loading = currentPage?.renderingState !== RenderingStates.FINISHED;
	PDFViewerApplication.toolbar.updateLoadingIndicatorState(loading);
}

function webViewerResize() {
	const { pdfDocument, pdfViewer } = PDFViewerApplication;

	pdfViewer.updateContainerHeightCss();

	if (!pdfDocument) {
		return;
	}
	const currentScaleValue = pdfViewer.currentScaleValue;
	if (
		currentScaleValue === 'auto' ||
		currentScaleValue === 'page-fit' ||
		currentScaleValue === 'page-width'
	) {
		// Note: the scale is constant for 'page-actual'.
		pdfViewer.currentScaleValue = currentScaleValue;
	}
	pdfViewer.update();

	pdfembEvaluateWantMobile();
}

const WebViewInitialized = () => {
	let fileUrl;
	const queryString = document.location.search.substring( 1 );
	const params = parseQueryString( queryString );
	fileUrl = params.get( 'pdfemb-serveurl' ) ?? params.get( 'url' );

	// This is NOT a secured file.
	if ( fileUrl.search( '/securepdfs/' ) === - 1 ) {
		PDFViewerApplication.open( fileUrl );
	}
	else {
		// Right click is disabled.
		if ( pdfemb_trans.disablerightclick !== undefined && pdfemb_trans.disablerightclick === true ) {
			document.oncontextmenu = function() {
				return false;
			};
		}

		pdfembGetPDF( document.location.href );
	}
};

// Triggered on resize, opening DevTools, opening fullscreen mode, etc.
function webViewerScaleChanging(evt) {

	// This is valid only for the initial load in fullscreen.
	if ( PDFViewerApplication.isFullscreen() ) {
		let fpZoomValue = isNaN( pdfemb_trans.fpzoom ) ? pdfemb_trans.fpzoom : parseInt( pdfemb_trans.fpzoom, 10 ) / 100;

		PDFViewerApplication.toolbar.setPageScale( fpZoomValue );
		// Reset the fullscreen so the user can interact with the zoom options (changing the scale manually).
		PDFViewerApplication.resetFullscreen();
	} else {
		PDFViewerApplication.toolbar.setPageScale( evt.presetValue, evt.scale );
	}

	PDFViewerApplication.pdfViewer.update();
}

function webViewerRotationChanging(evt) {
	PDFViewerApplication.pdfThumbnailViewer.pagesRotation = evt.pagesRotation;

	// TODO: Method not yet implemented.
	// PDFViewerApplication.forceRendering();

	// Ensure that the active page doesn't change during rotation.
	PDFViewerApplication.pdfViewer.currentPageNumber = evt.pageNumber;
}

function webViewerPageChanging({ pageNumber, pageLabel }) {
	PDFViewerApplication.toolbar.setPageNumber(pageNumber, pageLabel);
}

function webViewerResolutionChange(evt) {
	PDFViewerApplication.pdfViewer.refresh();
}
function webViewerPresentationModeChanged(evt) {
	PDFViewerApplication.pdfViewer.presentationModeState = evt.state;
}
function webViewerSwitchAnnotationEditorMode(evt) {
	PDFViewerApplication.pdfViewer.annotationEditorMode = evt.mode;
}
function webViewerPrint() {
	PDFViewerApplication.triggerPrinting();
}
function webViewerFullscreen() {
	PDFViewerApplication.goFullscreen();
}
function webViewerDownload() {
	PDFViewerApplication.download();
}
function webViewerFirstPage() {
	if (PDFViewerApplication.pdfDocument) {
		PDFViewerApplication.page = 1;
	}
}
function webViewerLastPage() {
	if (PDFViewerApplication.pdfDocument) {
		PDFViewerApplication.page = PDFViewerApplication.pagesCount;
	}
}
function webViewerSwitchAnnotationEditorParams(evt) {
	PDFViewerApplication.pdfViewer.annotationEditorParams = evt;
}
function webViewerNextPage() {
	PDFViewerApplication.pdfViewer.nextPage();

	if ( pdfemb_trans.scrolltotop ) {
		PDFViewerApplication.appConfig.appContainer.scrollIntoView();
	}
}
function webViewerPreviousPage() {
	PDFViewerApplication.pdfViewer.previousPage();
}
function webViewerZoomIn() {
	PDFViewerApplication.zoomIn();
}
function webViewerZoomOut() {
	PDFViewerApplication.zoomOut();
}
function webViewerZoomReset() {
	PDFViewerApplication.zoomReset();
}
function webViewerNamedAction(evt) {
	// Processing a couple of named actions that might be useful, see also
	// `PDFLinkService.executeNamedAction`.
	switch (evt.action) {
		case 'GoToPage':
			PDFViewerApplication.appConfig.toolbar.pageNumber.select();
			break;

		case 'Find':
			if (!PDFViewerApplication.supportsIntegratedFind) {
				PDFViewerApplication.findBar.toggle();
			}
			break;

		case 'Print':
			PDFViewerApplication.triggerPrinting();
			break;

		case 'SaveAs':
			PDFViewerApplication.downloadOrSave();
			break;
	}
}
function webViewerPresentationMode() {
	PDFViewerApplication.requestPresentationMode();
}

let zoomDisabledTimeout = null;
function setZoomDisabledTimeout() {
	if (zoomDisabledTimeout) {
		clearTimeout(zoomDisabledTimeout);
	}
	zoomDisabledTimeout = setTimeout(function () {
		zoomDisabledTimeout = null;
	}, WHEEL_ZOOM_DISABLED_TIMEOUT);
}

function webViewerPageNumberChanged(evt) {
	const pdfViewer = PDFViewerApplication.pdfViewer;
	// Note that for `<input type="number">` HTML elements, an empty string will
	// be returned for non-number inputs; hence we simply do nothing in that case.
	if (evt.value !== '') {
		PDFViewerApplication.pdfLinkService.goToPage(evt.value);
	}

	// Ensure that the page number input displays the correct value, even if the
	// value entered by the user was invalid (e.g. a floating point number).
	if (
		evt.value !== pdfViewer.currentPageNumber.toString() &&
		evt.value !== pdfViewer.currentPageLabel
	) {
		PDFViewerApplication.toolbar.setPageNumber(
			pdfViewer.currentPageNumber,
			pdfViewer.currentPageLabel,
		);
	}
}
function webViewerScrollModeChanged(evt) {
	if (PDFViewerApplication.isInitialViewSet) {
		// Only update the storage when the document has been loaded *and* rendered.
		PDFViewerApplication.store?.set('scrollMode', evt.mode).catch(() => {
			// Unable to write to storage.
		});
	}
}

function webViewerUpdateFindMatchesCount( { matchesCount } ) {
	if ( PDFViewerApplication.supportsIntegratedFind ) {
		PDFViewerApplication.externalServices.updateFindMatchesCount( matchesCount );
	}
	else {
		PDFViewerApplication.findBar.updateResultsCount( matchesCount );
	}
}

function webViewerWheel(evt) {
	const { pdfViewer, supportedMouseWheelZoomModifierKeys } =
		PDFViewerApplication;

	if (pdfViewer.isInPresentationMode) {
		return;
	}

	if (
		(evt.ctrlKey && supportedMouseWheelZoomModifierKeys.ctrlKey) ||
		(evt.metaKey && supportedMouseWheelZoomModifierKeys.metaKey)
	) {
		// Only zoom the pages, not the entire viewer.
		evt.preventDefault();
		// NOTE: this check must be placed *after* preventDefault.
		if (zoomDisabledTimeout || document.visibilityState === 'hidden') {
			return;
		}

		// It is important that we query deltaMode before delta{X,Y}, so that
		// Firefox doesn't switch to DOM_DELTA_PIXEL mode for compat with other
		// browsers, see https://bugzilla.mozilla.org/show_bug.cgi?id=1392460.
		const deltaMode = evt.deltaMode;
		const delta = normalizeWheelEventDirection(evt);
		const previousScale = pdfViewer.currentScale;

		let ticks = 0;
		if (
			deltaMode === WheelEvent.DOM_DELTA_LINE ||
			deltaMode === WheelEvent.DOM_DELTA_PAGE
		) {
			// For line-based devices, use one tick per event, because different
			// OSs have different defaults for the number lines. But we generally
			// want one "clicky" roll of the wheel (which produces one event) to
			// adjust the zoom by one step.
			if (Math.abs(delta) >= 1) {
				ticks = Math.sign(delta);
			} else {
				// If we're getting fractional lines (I can't think of a scenario
				// this might actually happen), be safe and use the accumulator.
				ticks = PDFViewerApplication.accumulateWheelTicks(delta);
			}
		} else {
			// pixel-based devices
			const PIXELS_PER_LINE_SCALE = 30;
			ticks = PDFViewerApplication.accumulateWheelTicks( delta / PIXELS_PER_LINE_SCALE );
		}

		if (ticks < 0) {
			PDFViewerApplication.zoomOut(-ticks);
		} else if (ticks > 0) {
			PDFViewerApplication.zoomIn(ticks);
		}

		const currentScale = pdfViewer.currentScale;
		if (previousScale !== currentScale) {
			// After scaling the page via zoomIn/zoomOut, the position of the upper-
			// left corner is restored. When the mouse wheel is used, the position
			// under the cursor should be restored instead.
			const scaleCorrectionFactor = currentScale / previousScale - 1;
			const rect = pdfViewer.container.getBoundingClientRect();
			const dx = evt.clientX - rect.left;
			const dy = evt.clientY - rect.top;
			pdfViewer.container.scrollLeft += dx * scaleCorrectionFactor;
			pdfViewer.container.scrollTop += dy * scaleCorrectionFactor;
		}
	} else {
		setZoomDisabledTimeout();
	}
}
function webViewerTouchStart(evt) {
	if (evt.touches.length > 1) {
		// Disable touch-based zooming, because the entire UI bits gets zoomed and
		// that doesn't look great. If we do want to have a good touch-based
		// zooming experience, we need to implement smooth zoom capability (probably
		// using a CSS transform for faster visual response, followed by async
		// re-rendering at the final zoom level) and do gesture detection on the
		// touchmove events to drive it. Or if we want to settle for a less good
		// experience we can make the touchmove events drive the existing step-zoom
		// behaviour that the ctrl+mousewheel path takes.
		evt.preventDefault();
	}
}

function webViewerKeyDown(evt) {
	if (PDFViewerApplication.overlayManager.active) {
		return;
	}
	const { eventBus, pdfViewer } = PDFViewerApplication;
	const isViewerInPresentationMode = pdfViewer.isInPresentationMode;

	let handled = false,
		ensureViewerFocused = false;
	const cmd =
		(evt.ctrlKey ? 1 : 0) |
		(evt.altKey ? 2 : 0) |
		(evt.shiftKey ? 4 : 0) |
		(evt.metaKey ? 8 : 0);

	// First, handle the key bindings that are independent whether an input
	// control is selected or not.
	if (cmd === 1 || cmd === 8 || cmd === 5 || cmd === 12) {
		// either CTRL or META key with optional SHIFT.
		switch (evt.keyCode) {
			case 70: // f
				if (
					!PDFViewerApplication.supportsIntegratedFind &&
					!evt.shiftKey
				) {
					PDFViewerApplication.findBar.open();
					handled = true;
				}
				break;
			case 71: // g
				if (!PDFViewerApplication.supportsIntegratedFind) {
					const { state } = PDFViewerApplication.findController;
					if (state) {
						const eventState = Object.assign(
							Object.create(null),
							state,
							{
								source: window,
								type: 'again',
								findPrevious: cmd === 5 || cmd === 12,
							},
						);
						eventBus.dispatch('find', eventState);
					}
					handled = true;
				}
				break;
			case 61: // FF/Mac '='
			case 107: // FF '+' and '='
			case 187: // Chrome '+'
			case 171: // FF with German keyboard
				if (!isViewerInPresentationMode) {
					PDFViewerApplication.zoomIn();
				}
				handled = true;
				break;
			case 173: // FF/Mac '-'
			case 109: // FF '-'
			case 189: // Chrome '-'
				if (!isViewerInPresentationMode) {
					PDFViewerApplication.zoomOut();
				}
				handled = true;
				break;
			case 48: // '0'
			case 96: // '0' on Numpad of Swedish keyboard
				if (!isViewerInPresentationMode) {
					// keeping it unhandled (to restore page zoom to 100%)
					setTimeout(function () {
						// ... and resetting the scale after browser adjusts its scale
						PDFViewerApplication.zoomReset();
					});
					handled = false;
				}
				break;

			case 38: // up arrow
				if (
					isViewerInPresentationMode ||
					PDFViewerApplication.page > 1
				) {
					PDFViewerApplication.page = 1;
					handled = true;
					ensureViewerFocused = true;
				}
				break;
			case 40: // down arrow
				if (
					isViewerInPresentationMode ||
					PDFViewerApplication.page < PDFViewerApplication.pagesCount
				) {
					PDFViewerApplication.page = PDFViewerApplication.pagesCount;
					handled = true;
					ensureViewerFocused = true;
				}
				break;
		}
	}

	if (typeof PDFJSDev === 'undefined' || PDFJSDev.test('GENERIC || CHROME')) {
		// CTRL or META without shift
		if (cmd === 1 || cmd === 8) {
			switch (evt.keyCode) {
				case 83: // s
					eventBus.dispatch('download', { source: window });
					handled = true;
					break;

				case 79: // o
					if (
						typeof PDFJSDev === 'undefined' ||
						PDFJSDev.test('GENERIC')
					) {
						eventBus.dispatch('openfile', { source: window });
						handled = true;
					}
					break;
			}
		}
	}

	// CTRL+ALT or Option+Command
	if (cmd === 3 || cmd === 10) {
		switch (evt.keyCode) {
			case 80: // p
				PDFViewerApplication.requestPresentationMode();
				handled = true;
				break;
			case 71: // g
				// focuses input#pageNumber field
				PDFViewerApplication.appConfig.toolbar.pageNumber.select();
				handled = true;
				break;
		}
	}

	if (handled) {
		if (ensureViewerFocused && !isViewerInPresentationMode) {
			pdfViewer.focus();
		}
		evt.preventDefault();
		return;
	}

	// Some shortcuts should not get handled if a control/input element
	// is selected.
	const curElement = getActiveOrFocusedElement();
	const curElementTagName = curElement?.tagName.toUpperCase();
	if (
		curElementTagName === 'INPUT' ||
		curElementTagName === 'TEXTAREA' ||
		curElementTagName === 'SELECT' ||
		curElement?.isContentEditable
	) {
		// Make sure that the secondary toolbar is closed when Escape is pressed.
		if (evt.keyCode !== /* Esc = */ 27) {
			return;
		}
	}

	// No control key pressed at all.
	if (cmd === 0) {
		let turnPage = 0,
			turnOnlyIfPageFit = false;
		switch (evt.keyCode) {
			case 38: // up arrow
			case 33: // pg up
				// vertical scrolling using arrow/pg keys
				if (pdfViewer.isVerticalScrollbarEnabled) {
					turnOnlyIfPageFit = true;
				}
				turnPage = -1;
				break;
			case 8: // backspace
				if (!isViewerInPresentationMode) {
					turnOnlyIfPageFit = true;
				}
				turnPage = -1;
				break;
			case 37: // left arrow
				// horizontal scrolling using arrow keys
				if (pdfViewer.isHorizontalScrollbarEnabled) {
					turnOnlyIfPageFit = true;
				}
			/* falls through */
			case 75: // 'k'
			case 80: // 'p'
				turnPage = -1;
				break;
			case 27: // esc key
				if (
					!PDFViewerApplication.supportsIntegratedFind &&
					PDFViewerApplication.findBar.opened
				) {
					PDFViewerApplication.findBar.close();
					handled = true;
				}
				break;
			case 40: // down arrow
			case 34: // pg down
				// vertical scrolling using arrow/pg keys
				if (pdfViewer.isVerticalScrollbarEnabled) {
					turnOnlyIfPageFit = true;
				}
				turnPage = 1;
				break;
			case 13: // enter key
			case 32: // spacebar
				if (!isViewerInPresentationMode) {
					turnOnlyIfPageFit = true;
				}
				turnPage = 1;
				break;
			case 39: // right arrow
				// horizontal scrolling using arrow keys
				if (pdfViewer.isHorizontalScrollbarEnabled) {
					turnOnlyIfPageFit = true;
				}
			/* falls through */
			case 74: // 'j'
			case 78: // 'n'
				turnPage = 1;
				break;

			case 36: // home
				if (
					isViewerInPresentationMode ||
					PDFViewerApplication.page > 1
				) {
					PDFViewerApplication.page = 1;
					handled = true;
					ensureViewerFocused = true;
				}
				break;
			case 35: // end
				if (
					isViewerInPresentationMode ||
					PDFViewerApplication.page < PDFViewerApplication.pagesCount
				) {
					PDFViewerApplication.page = PDFViewerApplication.pagesCount;
					handled = true;
					ensureViewerFocused = true;
				}
				break;

			case 83: // 's'
				PDFViewerApplication.pdfCursorTools.switchTool(
					CursorTool.SELECT,
				);
				break;
			case 72: // 'h'
				PDFViewerApplication.pdfCursorTools.switchTool(CursorTool.HAND);
				break;

			case 82: // 'r'
				PDFViewerApplication.rotatePages(90);
				break;
		}

		if (
			turnPage !== 0 &&
			(!turnOnlyIfPageFit || pdfViewer.currentScaleValue === 'page-fit')
		) {
			if (turnPage > 0) {
				pdfViewer.nextPage();
			} else {
				pdfViewer.previousPage();
			}
			handled = true;
		}
	}

	// shift-key
	if (cmd === 4) {
		switch (evt.keyCode) {
			case 13: // enter key
			case 32: // spacebar
				if (
					!isViewerInPresentationMode &&
					pdfViewer.currentScaleValue !== 'page-fit'
				) {
					break;
				}
				pdfViewer.previousPage();

				handled = true;
				break;

			case 82: // 'r'
				PDFViewerApplication.rotatePages(-90);
				break;
		}
	}

	if (!handled && !isViewerInPresentationMode) {
		// 33=Page Up  34=Page Down  35=End    36=Home
		// 37=Left     38=Up         39=Right  40=Down
		// 32=Spacebar
		if (
			(evt.keyCode >= 33 && evt.keyCode <= 40) ||
			(evt.keyCode === 32 && curElementTagName !== 'BUTTON')
		) {
			ensureViewerFocused = true;
		}
	}

	if (ensureViewerFocused && !pdfViewer.containsElement(curElement)) {
		// The page container is not focused, but a page navigation key has been
		// pressed. Change the focus to the viewer container to make sure that
		// navigation by keyboard works as expected.
		pdfViewer.focus();
	}

	if (handled) {
		evt.preventDefault();
	}
}

function webViewerUpdateFindControlState({
	state,
	previous,
	matchesCount,
	rawQuery,
}) {
	if (PDFViewerApplication.supportsIntegratedFind) {
		PDFViewerApplication.externalServices.updateFindControlState({
			result: state,
			findPrevious: previous,
			matchesCount,
			rawQuery,
		});
	} else {
		PDFViewerApplication.findBar.updateUIState(
			state,
			previous,
			matchesCount,
		);
	}
}

function webViewerFindFromUrlHash(evt) {
	PDFViewerApplication.eventBus.dispatch('find', {
		source: evt.source,
		type: '',
		query: evt.query,
		phraseSearch: evt.phraseSearch,
		caseSensitive: false,
		entireWord: false,
		highlightAll: true,
		findPrevious: false,
		matchDiacritics: true,
	});
}
function webViewerSpreadModeChanged(evt) {
	if (PDFViewerApplication.isInitialViewSet) {
		// Only update the storage when the document has been loaded *and* rendered.
		PDFViewerApplication.store?.set('spreadMode', evt.mode).catch(() => {
			// Unable to write to storage.
		});
	}
}
function webViewerScaleChanged(evt) {
	PDFViewerApplication.pdfViewer.currentScaleValue = evt.value;
}
function webViewerRotateCw() {
	PDFViewerApplication.rotatePages(90);
}
function webViewerRotateCcw() {
	PDFViewerApplication.rotatePages(-90);
}
function webViewerOptionalContentConfig(evt) {
	PDFViewerApplication.pdfViewer.optionalContentConfigPromise = evt.promise;
}
function webViewerSwitchScrollMode(evt) {
	PDFViewerApplication.pdfViewer.scrollMode = evt.mode;
}
function webViewerSwitchSpreadMode(evt) {
	PDFViewerApplication.pdfViewer.spreadMode = evt.mode;
}
function webViewerDocumentProperties() {
	PDFViewerApplication.pdfDocumentProperties.open();
}
function webViewerVisibilityChange(evt) {
	if (document.visibilityState === 'visible') {
		// Ignore mouse wheel zooming during tab switches (bug 1503412).
		setZoomDisabledTimeout();
	}
}

window.PDFViewerApplication = PDFViewerApplication;

/**
 * Track the PDF file actions like view and download.
 */
function pdfembTrackPDF( method ) {

	if ( pdfemb_trans.tracking === false ) {
		return;
	}

	let data = PDFViewerApplication.appConfig.dataContainer;

	jQuery.post(
		pdfemb_trans.ajaxurl,
		{
			action: 'pdfemb_tracking',
			method: method,
			nonce: pdfemb_trans.trackingNonce,
			url: data.getAttribute( 'data-file' ),
			id: data.getAttribute( 'data-pdfid' ),
		}
	);
}

/**
 *  Dynamically load the secured PDF file.
 */
function pdfembGetPDF( url ) {

	let nonce = '';

	if ( pdfemb_trans.secureNonce !== "undefined" && pdfemb_trans.secureNonce !== '' ){
		nonce = '&pdfemb-nonce=' + pdfemb_trans.secureNonce;
	}

	url = url + nonce;

	pdfembAddAjaxBufferTransport();

	jQuery
		.ajax( {
			dataType: 'arraybuffer',
			type: 'GET',
			url: url,
			processData:false,
		} )
		.done( function( blob ) {

			let args = {
				secure: true,
				nonced_url: url,
			};

			pdfemb_rc4ab( blob );

			PDFViewerApplication.open( new Uint8Array( blob ), args );
		} )
		.fail( function( jqXHR, textStatus, errorThrown ) {
			return false;
		} );
}

function pdfembAddAjaxBufferTransport() {
	let pdfembAddAjaxBufferTransport_added = false;

	if (pdfembAddAjaxBufferTransport_added) {
		return;
	}
	pdfembAddAjaxBufferTransport_added = true;

	// https://blog.artandlogic.com/ckeefer/2013-11/jquery-ajax-blobs
	// https://gist.github.com/SaneMethod/7548768/
	/**
	 * Register ajax transports for blob send/receive and array buffer send/receive via XMLHttpRequest Level 2
	 * within the comfortable framework of the jquery ajax request, with full support for promises.
	 *
	 * Notice the +* in the dataType string? The + indicates we want this transport to be prepended to the list
	 * of potential transports (so it gets first dibs if the request passes the conditions within to provide the
	 * ajax transport, preventing the standard transport from hogging the request), and the * indicates that
	 * potentially any request with any dataType might want to use the transports provided herein.
	 *
	 * Remember to specify 'processData:false' in the ajax options when attempting to send a blob or arraybuffer -
	 * otherwise jquery will try (and fail) to convert the blob or buffer into a query string.
	 */
	jQuery.ajaxTransport('+*', function (options, originalOptions, jqXHR) {
		// Test for the conditions that mean we can/want to send/receive blobs or arraybuffers - we need XMLHttpRequest
		// level 2 (so feature-detect against window.FormData), feature detect against window.Blob or window.ArrayBuffer,
		// and then check to see if the dataType is blob/arraybuffer or the data itself is a Blob/ArrayBuffer
		if (
			window.FormData &&
			(
				( options.dataType && (options.dataType === 'blob' || options.dataType === 'arraybuffer') ) ||
				(
					options.data &&
					(
						(window.Blob && options.data instanceof Blob) ||
						(window.ArrayBuffer && options.data instanceof ArrayBuffer)
					)
				)
			)
		) {
			return {
				/**
				 * Return a transport capable of sending and/or receiving blobs - in this case, we instantiate
				 * a new XMLHttpRequest and use it to actually perform the request, and funnel the result back
				 * into the jquery complete callback (such as the success function, done blocks, etc.)
				 *
				 * @param headers
				 * @param completeCallback
				 */
				send: function (headers, completeCallback) {
					var xhr = new XMLHttpRequest(),
						url = options.url || window.location.href,
						type = options.type || 'GET',
						dataType = options.dataType || 'text',
						data = options.data || null,
						async = options.async || true;

					xhr.addEventListener('load', function () {
						var res = {};

						res[dataType] = xhr.response;
						completeCallback(
							xhr.status,
							xhr.statusText,
							res,
							xhr.getAllResponseHeaders(),
						);
					});

					xhr.open(type, url, async);
					xhr.responseType = dataType;
					xhr.send(data);
				},
				abort: function () {
					jqXHR.abort();
				},
			};
		}
	});
}
function pdfemb_rc4ab(ab) {
	var s = [],
		j = 0,
		x,
		res = '',
		input,
		output;
	var dv = new DataView(ab);

	// Check for Unicode BOM and skip it
	var starty = 0;
	if (
		dv.getUint8(0) == 0xef &&
		dv.getUint8(1) == 0xbb &&
		dv.getUint8(2) == 0xbf
	) {
		starty = 3;
	}

	// Decrypt
	for (var i = 0; i < 256; i++) {
		s[i] = i;
	}
	for (i = 0; i < 256; i++) {
		j = (j + s[i] + pdfemb_trans.k.charCodeAt(i % pdfemb_trans.k.length)) % 256;
		x = s[i];
		s[i] = s[j];
		s[j] = x;
	}
	i = 0;
	j = 0;
	for (var y = starty; y < ab.byteLength; y++) {
		i = (i + 1) % 256;
		j = (j + s[i]) % 256;
		x = s[i];
		s[i] = s[j];
		s[j] = x;
		input = dv.getUint8(y);
		output = input ^ s[(s[i] + s[j]) % 256];
		dv.setUint8(y, output);
	}
}

/**
 * Check the current width and make decision whether to show/hide the wantmobile button.
 */
function pdfembEvaluateWantMobile() {
	let pdf_target_id = document.getElementsByClassName( 'wppdf-fullscreen-button' )[ 0 ].getAttribute( 'data-pdf-id' );
	let frame = window.parent.document.getElementById( 'wppdf-emb-iframe-' + pdf_target_id );

	// Show/hide the wantmobile section.
	if ( pdfemb_trans.mobilewidth > PDFViewerApplication.appConfig.mainContainer.clientWidth && ! frame.classList.contains( 'wppdf-fullscreen' ) ) {
		$( '.pdfemb-inner-div-wantmobile-fswrap' ).show();
	} else {
		$( '.pdfemb-inner-div-wantmobile-fswrap' ).hide();
	}
}

function renderWatermark( ctx, pageNum, zoom ) {
	let watermark_map = pdfemb_trans.watermark_map;

	if ( ! Array.isArray( watermark_map ) || watermark_map.length === 0 ) {
		return;
	}

	if (
		typeof pdfemb_trans.watermark_evenpagesonly !== 'undefined' &&
		pdfemb_trans.watermark_evenpagesonly &&
		pageNum % 2 !== 0
	) {
		return;
	}

	var xmargin = 5,
		y = 10;
	var xalign = 'center';
	var rotate = 45;
	var fontsize = 20;
	for ( var i = 0; i < watermark_map.length; ++ i ) {
		var thisMap = watermark_map[ i ];
		if ( Array.isArray( thisMap ) && thisMap.length >= 1 ) {
			var text = thisMap[ 0 ];
			if ( ! ctx ) {
				break;
			}

			ctx.save();

			ctx.lineWidth = 1;
			ctx.fillStyle = '#444444';
			ctx.lineStyle = '#111111';
			ctx.textAlign = 'left';
			ctx.font = '20pt sans-serif';
			ctx.globalAlpha = 0.1;
			if ( thisMap.length >= 2 ) {
				xmargin = thisMap[ 1 ];

				if ( thisMap.length >= 3 ) {
					y = thisMap[ 2 ];

					if ( thisMap.length >= 4 ) {
						xalign = thisMap[ 3 ];

						if ( thisMap.length >= 5 ) {
							rotate = thisMap[ 4 ];

							if ( thisMap.length >= 6 ) {
								fontsize = thisMap[ 5 ];

								ctx.font = Math.round( 100 * fontsize * (ctx.canvas.width / 1000) ) / 100 + 'pt sans-serif';

								if (
									thisMap.length >= 7 &&
									typeof thisMap[ 6 ] == 'object'
								) {
									var styleList = thisMap[ 6 ];
									for ( var prop in styleList ) {
										ctx[ prop ] = styleList[ prop ];
									}
								}
							}
						}
					}
				}
			}

			var mtwidth = ctx.measureText( text ).width;
			var mtheight = 10;

			var drawx,
				drawy = (ctx.canvas.height * y) / 100;
			var textx, texty;

			if ( xalign === 'left' ) {
				drawx = (ctx.canvas.width * xmargin) / 100;
				textx = 0;
				texty = 0;
			}
			else if ( xalign === 'right' ) {
				drawx = ctx.canvas.width * (1 - xmargin / 100);
				textx = - mtwidth;
				texty = 0;
			}
			else {
				drawx = ctx.canvas.width / 2;
				textx = - mtwidth / 2;
				texty = 0;
			}

			ctx.translate( drawx, drawy );

			ctx.rotate( (- Math.PI * rotate) / 360 );

			ctx.fillText( text, textx, texty );

			ctx.restore();

			y += 10;
		}
	}
}

export { PDFViewerApplication };
