<?php

namespace PDFEmbedder\Premium;

/**
 * The plugin requires PDF Embedder Lite to be installed and activated.
 * We don't check for PHP version because PDF Embedder Lite already does that.
 *
 * @since 5.2.0
 */
final class Requirements {

	const MIN_LITE_VERSION = '4.8.0';

	const LITE_PLUGIN_SLUG = 'pdf-embedder/pdf_embedder.php';
	const LITE_PLUGIN_BASE = 'pdf-embedder';

	/**
	 * The key of the missing requirement.
	 *
	 * @since 5.2.0
	 *
	 * @var string
	 */
	protected $missing = '';

	/**
	 * Check if all requirements are met.
	 *
	 * @since 5.2.0
	 */
	public function are_not_met(): bool {

		if ( ! function_exists( 'is_plugin_active' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		// If not installed or not activated.
		if ( ! is_plugin_active( self::LITE_PLUGIN_SLUG ) ) {

			// If installed, but not activated.
			if ( file_exists( WP_PLUGIN_DIR . '/' . self::LITE_PLUGIN_SLUG ) ) {
				$this->missing = 'not_activated_lite';
			} else {
				// Not even installed.
				$this->missing = 'not_installed_lite';
			}

			return true;
		}

		// If older version is activated.
		$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/' . self::LITE_PLUGIN_SLUG, false, false );

		if ( version_compare( $plugin_data['Version'], self::MIN_LITE_VERSION, '<' ) ) {
			$this->missing = 'old_active_lite';

			return true;
		}

		return false;
	}

	/**
	 * Notify the user about missing requirements.
	 *
	 * @since 5.2.0
	 */
	public function notify() {

		switch ( $this->missing ) {
			case 'not_installed_lite':
				$this->notice_not_installed_lite();
				break;

			case 'not_activated_lite':
				$this->notice_not_activated_lite();
				break;

			case 'old_active_lite':
				$this->notice_old_active_lite();
				break;
		}
	}

	/**
	 * Display a notice when PDF Embedder Lite is not installed and not activated.
	 *
	 * @since 5.2.0
	 */
	private function notice_not_installed_lite() { // phpcs:ignore WPForms.PHP.HooksMethod.InvalidPlaceForAddingHooks

		add_action(
			'admin_notices',
			static function () {
				?>
				<div class="notice notice-error">
					<p>
						<?php
						esc_html_e( 'PDF Embedder Premium requires PDF Embedder Lite to be installed and activated.', 'pdf-embedder-premium' );
						echo '<br>';

						if ( current_user_can( 'install_plugins' ) ) {
							$action_url = wp_nonce_url(
								add_query_arg(
									[
										'action' => 'install-plugin',
										'plugin' => self::LITE_PLUGIN_BASE,
									],
									admin_url( 'update.php' )
								),
								'install-plugin_' . self::LITE_PLUGIN_BASE
							);

							printf(
								wp_kses( /* translators: %s - PDF Embedder Lite install URL. */
									__( 'Please <a href="%s">install and activate</a> the Lite version to continue.', 'pdf-embedder-premium' ),
									[
										'a' => [
											'href' => [],
										],
									]
								),
								esc_url( $action_url )
							);
						} else {
							esc_html_e( 'Please install and activate the Lite version to continue.', 'pdf-embedder-premium' );
						}
						?>
					</p>
				</div>
				<?php
			}
		);
	}

	/**
	 * Display a notice when PDF Embedder Lite is installed but not activated.
	 *
	 * @since 5.2.0
	 */
	private function notice_not_activated_lite() { // phpcs:ignore WPForms.PHP.HooksMethod.InvalidPlaceForAddingHooks

		add_action(
			'admin_notices',
			static function () {
				?>
				<div class="notice notice-error">
					<p>
						<?php
						esc_html_e( 'PDF Embedder Premium requires PDF Embedder Lite to be activated.', 'pdf-embedder-premium' );
						echo '<br>';

						if ( current_user_can( 'activate_plugins' ) ) {
							$action_url = wp_nonce_url(
								add_query_arg(
									[
										'action' => 'activate',
										'plugin' => rawurlencode( self::LITE_PLUGIN_SLUG ),
									],
									admin_url( 'plugins.php' )
								),
								'activate-plugin_' . self::LITE_PLUGIN_SLUG
							);

							printf(
								wp_kses( /* translators: %s - PDF Embedder Lite activate URL. */
									__( 'Please <a href="%s">activate</a> the Lite version to continue.', 'pdf-embedder-premium' ),
									[
										'a' => [
											'href' => [],
										],
									]
								),
								esc_url( $action_url )
							);
						} else {
							esc_html_e( 'Please activate the Lite version to continue.', 'pdf-embedder-premium' );
						}
						?>
					</p>
				</div>
				<?php
			}
		);
	}

	/**
	 * Display a notice when PDF Embedder Lite is installed but an older version is activated.
	 *
	 * @since 5.2.0
	 */
	private function notice_old_active_lite() {

		add_action(
			'admin_notices',
			static function () {
				?>
				<div class="notice notice-error">
					<p>
						<?php
						printf( /* translators: %s - PDF Embedder Lite version. */
							esc_html__( 'PDF Embedder Premium requires at least PDF Embedder Lite %s to work properly.', 'pdf-embedder-premium' ),
							esc_html( self::MIN_LITE_VERSION )
						);
						echo '<br>';

						if ( current_user_can( 'update_plugins' ) ) {
							$action_url = wp_nonce_url(
								add_query_arg(
									[
										'action' => 'upgrade-plugin',
										'plugin' => rawurlencode( self::LITE_PLUGIN_SLUG ),
									],
									self_admin_url( 'update.php' )
								),
								'upgrade-plugin_' . self::LITE_PLUGIN_SLUG
							);

							printf(
								wp_kses( /* translators: %s - PDF Embedder Lite activate URL. */
									__( 'Please <a href="%s">update</a> the Lite version to continue.', 'pdf-embedder-premium' ),
									[
										'a' => [
											'href' => [],
										],
									]
								),
								esc_url( $action_url )
							);
						} else {
							esc_html_e( 'Please update the Lite version to continue.', 'pdf-embedder-premium' );
						}
						?>
					</p>
				</div>
				<?php
			}
		);
	}
}
