<?php

namespace PDFEmbedder\Premium\Admin;

use WP_Post;
use PDFEmbedder\Options;
use PDFEmbedder\Helpers\Multisite;

/**
 * Admin page.
 *
 * @since 5.2.0
 */
class Admin {

	/**
	 * Assign all hooks to proper places.
	 *
	 * @since 5.2.0
	 */
	public function hooks() {

		// Plugins page.
		add_filter( Multisite::is_network_activated() ? 'network_admin_plugin_action_links' : 'plugin_action_links', [ $this, 'register_plugin_action_links' ], 20, 2 );

		// Remove the default educational tab content in Lite.
		remove_action( 'pdfemb_admin_settings_render_section_mobile', [ pdf_embedder()->admin(), 'render_mobile' ] );

		remove_action( 'pdfemb_admin_settings_extra', [ pdf_embedder()->admin(), 'render_ut_setting' ], 0 );

		// Media single item field.
		add_filter( 'attachment_fields_to_edit', [ $this, 'attachment_fields_to_edit' ], 20, 2 );
		add_action(
			'add_meta_boxes_attachment',
			function () {
				add_meta_box( 'attachment_meta_box', __( 'PDF Embedder', 'pdf-embedder-premium' ), [ $this, 'attachment_show_data_meta_box' ], 'attachment', 'side' );
			}
		);
		add_action( 'edit_attachment', [ $this, 'attachment_process_meta_box_data' ] );

		// Extending block with premium functionality.
		add_action(
			'enqueue_block_editor_assets',
			static function () {
				if ( ! is_admin() ) {
					return;
				}

				foreach ( pdf_embedder()->premium()->plans()->get() as $plan ) {
					$name = $plan::NAME;

					$index = PDFEMB_PREMIUM_PLUGIN_DIR . "block/{$name}/build/index.asset.php";

					if ( ! file_exists( $index ) ) {
						continue;
					}

					$config = require $index;

					// That's a handle for the Lite block generated by WP itself.
					// As we require Lite block, let's put a hard dependency on it.
					$config['dependencies'][] = 'pdfemb-pdf-embedder-viewer-editor-script';

					wp_enqueue_script(
						"pdfemb-extend-block-$name",
						PDFEMB_PREMIUM_PLUGIN_URL . "block/$name/build/index.js",
						$config['dependencies'],
						$config['version']
					);
				}
			}
		);
	}

	/**
	 * Register plugin action links.
	 *
	 * @since 5.2.0
	 *
	 * @param array  $links Plugin action links.
	 * @param string $file  Plugin file.
	 */
	public function register_plugin_action_links( $links, $file ): array {

		// Do not allow disabling the Lite plugin.
		if ( plugin_basename( PDFEMB_PLUGIN_FILE ) === $file ) {
			// These were set in Lite and no longer needed.
			unset(
				$links['premium'],
				$links['secure']
			);

			global $wp_version;

			if ( version_compare( $wp_version, '6.5.0', '<' ) ) {
				$links['deactivate'] = '<span style="color: #32373c">' . esc_html__( 'Required by PDF Embedder Premium', 'pdf-embedder-premium' ) . '</span>';
			}
		}

		return $links;
	}

	/**
	 * Add additional fields to the "Attachment details" media popup/screen.
	 *
	 * @since 5.2.0
	 *
	 * @param array   $form_fields An array of attachment form fields.
	 * @param WP_Post $attachment  The WP_Post attachment object.
	 */
	public function attachment_fields_to_edit( array $form_fields, WP_Post $attachment ): array { // phpcs:ignore Generic.Metrics.CyclomaticComplexity.TooHigh

		if ( $attachment->post_mime_type !== 'application/pdf' ) {
			return $form_fields;
		}

		$current_screen = get_current_screen();

		// We are in single attachment editing screen.
		if ( $current_screen && $current_screen->id === 'attachment' ) {
			// We have a special metabox for this, so we don't need to show the fields here.
			return $form_fields;
		}

		$options = pdf_embedder()->options()->get();

		if ( ! Options::is_on( $options['pdfemb_tracking'] ) ) {
			return $form_fields;
		}

		$downloads = get_post_meta( $attachment->ID, 'pdfemb-downloads', true );
		$views     = get_post_meta( $attachment->ID, 'pdfemb-views', true );
		$file_url  = wp_get_attachment_url( $attachment->ID );
		$is_secure = ! strpos( $file_url, '/securepdfs/' ) === false;

		if ( ! is_numeric( $downloads ) ) {
			$downloads = __( 'None', 'pdf-embedder-premium' );
		}

		if ( ! is_numeric( $views ) ) {
			$views = __( 'None', 'pdf-embedder-premium' );
		}

		$secured_value  = '<span class="dashicons dashicons-unlock pdfemb-admin-attachment-meta-icon"></span>' . esc_html__( 'No', 'pdf-embedder-premium' ) . '.&nbsp;';
		$secured_value .= sprintf(
							wp_kses( /* translators: %s - URL to the settings page. */
								__( '<a href="%s">Learn more</a>', 'pdf-embedder-premium' ),
								[
									'a' => [
										'href' => [],
									],
								]
							),
							esc_url( pdf_embedder()->admin()->get_settings_url( 'secure' ) )
						);

		if ( $is_secure ) {
			$secured_value = '<span class="dashicons dashicons-lock pdfemb-admin-attachment-meta-icon"></span>' . esc_html__( 'Yes', 'pdf-embedder-premium' );
		}

		$form_fields['pdfemb-secured'] = [
			'value' => $secured_value,
			'input' => 'value',
			'label' => __( 'PDF Secured', 'pdf-embedder-premium' ),
		];

		$form_fields['pdfemb-downloads'] = [
			'value' => $downloads,
			'input' => 'value',
			'label' => __( 'PDF Downloads', 'pdf-embedder-premium' ),
		];

		$form_fields['pdfemb-views'] = [
			'value' => $views,
			'input' => 'value',
			'label' => __( 'PDF Views', 'pdf-embedder-premium' ),
		];

		return $form_fields;
	}

	/**
	 * Show metabox with additional data for the attachment.
	 *
	 * @since 5.2.0
	 *
	 * @param WP_Post $attachment Post object.
	 */
	public function attachment_show_data_meta_box( WP_Post $attachment ) {

		if ( $attachment->post_mime_type !== 'application/pdf' ) {
			return;
		}

		$count_downloads = get_post_meta( $attachment->ID, 'pdfemb-downloads', true );
		$count_views     = get_post_meta( $attachment->ID, 'pdfemb-views', true );
		$file_url        = wp_get_attachment_url( $attachment->ID );
		$is_secure       = ! strpos( $file_url, '/securepdfs/' ) === false;

		if ( empty( $count_downloads ) ) {
			$count_downloads = 0;
		}

		if ( empty( $count_views ) ) {
			$count_views = 0;
		}
		?>

		<div class="attachment_field_containers">
			<p>
				<style>
					.pdfemb-admin-attachment-meta-icon {
						color: #8c8f94;
						display: inline-block;
						padding-right: 3px;
					}
				</style>

				<?php
				if ( $is_secure ) {
					echo '<span class="dashicons dashicons-lock pdfemb-admin-attachment-meta-icon"></span>';
					esc_html_e( 'Secured PDF', 'pdf-embedder-premium' );
				} else {
					echo '<span class="dashicons dashicons-unlock pdfemb-admin-attachment-meta-icon"></span>';

					esc_html_e( 'Not Secured PDF.', 'pdf-embedder-premium' );
					echo '&nbsp;';
					printf(
						wp_kses( /* translators: %s - URL to the settings page. */
							__( '<a href="%s">Learn more</a>', 'pdf-embedder-premium' ),
							[
								'a' => [
									'href' => [],
								],
							]
						),
						esc_url( pdf_embedder()->admin()->get_settings_url( 'secure' ) )
					);
				}
				?>
			</p>

			<p>
				<?php
				printf(
					/* translators: %d - number of downloads. */
					esc_html__( 'Downloads: %d', 'pdf-embedder-premium' ),
					(int) $count_downloads
				);
				?>
			</p>

			<button type="submit" name="pdfemb-action" value="reset-downloads" class="button">
				<?php esc_html_e( 'Reset Download Counter', 'pdf-embedder-premium' ); ?>
			</button>

			<p>
				<?php
				printf(
					/* translators: %d - number of views. */
					esc_html__( 'Views: %d', 'pdf-embedder-premium' ),
					(int) $count_views
				);
				?>
			</p>

			<button type="submit" name="pdfemb-action" value="reset-views" class="button">
				<?php esc_html_e( 'Reset View Counter', 'pdf-embedder-premium' ); ?>
			</button>
		</div>

		<?php
	}

	/**
	 * Process single attachment actions.
	 *
	 * @since 5.2.0
	 *
	 * @param int $attachment_id Attachment ID.
	 */
	public function attachment_process_meta_box_data( int $attachment_id ) {

		// phpcs:disable WordPress.Security.NonceVerification.Missing

		// Whether our "clear" button was clicked.
		if ( ! isset( $_POST['pdfemb-action'] ) ) {
			return;
		}

		// Check the user's permissions.
		if ( ! current_user_can( 'edit_post', $attachment_id ) ) {
			return;
		}

		switch ( sanitize_key( $_POST['pdfemb-action'] ) ) {
			case 'reset-downloads':
				update_post_meta( $attachment_id, 'pdfemb-downloads', 0 );
				break;

			case 'reset-views':
				update_post_meta( $attachment_id, 'pdfemb-views', 0 );
				break;
		}
		// phpcs:enable WordPress.Security.NonceVerification.Missing
	}
}
