<?php

namespace PDFEmbedder\Premium\Admin;

use stdClass;
use PDFEmbedder\Admin\License;
use PDFEmbedder\Helpers\Multisite;

/**
 * Class Updater to handle license checks and updates notifications.
 *
 * @since 5.2.0
 */
final class Updater {

	/**
	 * API URL.
	 *
	 * @since 5.2.0
	 *
	 * @var string
	 */
	const API_URL = 'https://wp-pdf.com/';

	/**
	 * License key.
	 *
	 * @since 5.2.0
	 *
	 * @var string
	 */
	protected $license_key = '';

	/**
	 * API Data.
	 *
	 * @since 5.2.0
	 *
	 * @var array
	 */
	private $api_data = [];

	/**
	 * Name: pdf-embedder-premium/pdf-embedder-premium.php.
	 *
	 * @since 5.2.0
	 *
	 * @var string
	 */
	private $name;

	/**
	 * Slug: pdf-embedder-premium.
	 *
	 * @since 5.2.0
	 *
	 * @var string
	 */
	private $slug;

	/**
	 * Warning Delay.
	 *
	 * @since 5.2.0
	 *
	 * @var int
	 */
	private $license_warning_delay = DAY_IN_SECONDS;

	/**
	 * Option value with the license status data.
	 * Content of the License::OPTION_STATUS option from DB.
	 *
	 * @since 5.2.0
	 *
	 * @var array
	 */
	protected $license_status_option = [];

	/**
	 * Display warnings.
	 *
	 * @since 5.2.0
	 *
	 * @var bool
	 */
	private $display_warnings = true;

	/**
	 * Cache Key.
	 *
	 * @since 5.2.0
	 *
	 * @var string
	 */
	private $cache_key;

	/**
	 * Version Info.
	 *
	 * @since 5.2.0
	 *
	 * @var mixed
	 */
	protected $version_info = null;

	/**
	 * Auto Check.
	 *
	 * @since 5.2.0
	 *
	 * @var bool
	 */
	protected $auto_license_checked = false;

	/**
	 * Class constructor.
	 *
	 * @since 5.2.0
	 *
	 * @param string $license_key Optional license key to work with.
	 */
	public function __construct( string $license_key = '' ) {

		$this->license_key = ! empty( $license_key ) ? sanitize_key( $license_key ) : License::get_key();

		$this->name = plugin_basename( PDFEMB_PREMIUM_PLUGIN_FILE );
		$this->slug = basename( PDFEMB_PREMIUM_PLUGIN_FILE, '.php' );

		// phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize
		$this->cache_key = md5( serialize( $this->slug . $this->license_key ) );
	}

	/**
	 * Set up WordPress filters to hook into WP's update process.
	 *
	 * @since 5.2.0
	 */
	public function hooks() {

		add_filter( 'pre_set_site_transient_update_plugins', [ $this, 'pre_set_site_transient_update_plugins_filter' ] );
		add_filter( 'plugins_api', [ $this, 'plugins_api_filter' ], 10, 3 );
		add_filter( 'http_request_args', [ $this, 'http_request_args' ], 10, 2 );
		add_action( 'admin_init', [ $this, 'admin_init' ] );
		add_action( 'admin_init', [ $this, 'show_changelog' ] );
	}

	/**
	 * Perform checks and display notices if needed.
	 *
	 * @since 5.2.0
	 */
	public function admin_init() { // phpcs:ignore Generic.Metrics.CyclomaticComplexity,WPForms.PHP.HooksMethod.InvalidPlaceForAddingHooks

		if ( ! current_user_can( Multisite::is_network_activated() ? 'manage_network_plugins' : 'update_plugins' ) ) {
			return;
		}

		// Setup hooks to display message if license is not valid.
		$license_status = $this->get_license_status_option();

		if (
			isset( $license_status['status'] ) &&
			isset( $license_status['result_cleared'] ) &&
			! $license_status['result_cleared']
		) {
			// Do they want it cleared?.
			// $nothanks_url = add_query_arg( License::OPTION_STATUS.'_eddsl_action', 'no_thanks' );.
			$queryname = License::OPTION_STATUS . '_eddsl_action';

			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			if ( isset( $_REQUEST[ $queryname ] ) && $_REQUEST[ $queryname ] === 'no_thanks' ) {
				$license_status['result_cleared'] = true;

				update_site_option( License::OPTION_STATUS, $license_status );

				$this->license_status_option = $license_status;
			} elseif ( $this->display_warnings ) {
				if ( Multisite::is_network_activated() ) {
					add_action( 'network_admin_notices', [ $this, 'eddsl_license_notice' ] );
				} else {
					add_action( 'admin_notices', [ $this, 'eddsl_license_notice' ] );
				}
			}
		}
	}

	/**
	 * Check for Updates at the defined API endpoint and modify the update array.
	 *
	 * This function dives into the update api just when WordPress creates its update array,
	 * then adds a custom API call and injects the custom plugin data retrieved from the API.
	 * It is reassembled from parts of the native WordPress plugin update code.
	 * See wp-includes/update.php line 121 for the original wp_update_plugins() function.
	 *
	 * @since 5.2.0
	 *
	 * @uses api_request()
	 *
	 * @param mixed $_transient_data Update array build by WordPress.
	 *
	 * @return mixed Modified data with custom plugin data.
	 */
	public function pre_set_site_transient_update_plugins_filter( $_transient_data ) { // phpcs:ignore Generic.Metrics.CyclomaticComplexity

		if ( ! is_object( $_transient_data ) ) {
			$_transient_data = new stdClass();
		}

		if ( empty( $_transient_data->response ) || empty( $_transient_data->response[ $this->name ] ) ) {

			if ( is_null( $this->version_info ) ) {
				$this->version_info = $this->api_request( 'get_version', [ 'slug' => $this->slug ] );
			}

			if (
				$this->version_info !== false &&
				is_object( $this->version_info ) &&
				isset( $this->version_info->new_version )
			) {

				if ( version_compare( PDFEMB_PREMIUM_VERSION, $this->version_info->new_version, '<' ) ) {
					$this->version_info->plugin               = $this->name;
					$_transient_data->response[ $this->name ] = $this->version_info;

				}

				$_transient_data->last_checked           = time();
				$_transient_data->checked[ $this->name ] = PDFEMB_PREMIUM_VERSION;
			}
		}

		// Do the actual license check now.
		if ( ! $this->auto_license_checked ) {

			// Returns only valid or invalid, but also contains expires and renewal_link.
			$api_check = $this->api_request( 'check_license', [ 'slug' => $this->slug ] );

			// Record latest status from license server.
			$license_status             = $this->update_license_status_option( $api_check );
			$this->auto_license_checked = true;

			// If we got invalid, then trying activating now.
			if ( isset( $license_status['status'] ) && ( $license_status['status'] === 'invalid' || $license_status['status'] === 'site_inactive' ) ) {
				$this->edd_license_activate(); // calls update_license_status_option.
			}
		}

		return $_transient_data;
	}

	/**
	 * Show update notification row - needed for multisite subsites, because WP won't tell you otherwise!
	 *
	 * @since 5.2.0
	 *
	 * @param string $file   Update file.
	 * @param array  $plugin Plugin.
	 */
	public function show_update_notification( $file, $plugin ) {

		if ( ! current_user_can( 'update_plugins' ) ) {
			return;
		}

		if ( ! is_multisite() ) {
			return;
		}

		if ( $this->name !== $file ) {
			return;
		}

		// Remove our filter on the site transient.
		remove_filter( 'pre_set_site_transient_update_plugins', [ $this, 'pre_set_site_transient_update_plugins_filter' ] );

		$update_cache = get_site_transient( 'update_plugins' );

		if ( ! is_object( $update_cache ) || empty( $update_cache->response ) || empty( $update_cache->response[ $this->name ] ) ) {

			$cache_key    = md5( 'edd_plugin_' . sanitize_key( $this->name ) . '_version_info-reg' );
			$version_info = $this->get_cached_version_info( $cache_key );

			if ( $version_info === false ) {

				$version_info = $this->api_request( 'get_version', [ 'slug' => $this->slug ] ); // 'plugin_latest_version'

				$this->set_version_info_cache( $version_info, $cache_key );
			}

			if ( ! is_object( $version_info ) ) {
				return;
			}

			if ( version_compare( PDFEMB_PREMIUM_VERSION, $version_info->new_version, '<' ) ) {

				$update_cache->response[ $this->name ] = $version_info;

			}

			$update_cache->last_checked           = time();
			$update_cache->checked[ $this->name ] = PDFEMB_PREMIUM_VERSION;

			set_site_transient( 'update_plugins', $update_cache );

		} else {

			$version_info = $update_cache->response[ $this->name ];

		}

		// Restore our filter.
		add_filter( 'pre_set_site_transient_update_plugins', [ $this, 'pre_set_site_transient_update_plugins_filter' ] );

		if (
			! empty( $update_cache->response[ $this->name ] ) &&
			version_compare( PDFEMB_PREMIUM_VERSION, $version_info->new_version, '<' )
		) {

			// build a plugin list row, with update notification.
			$wp_list_table = _get_list_table( 'WP_Plugins_List_Table' );

			echo '<tr class="plugin-update-tr"><td colspan="' . esc_attr( $wp_list_table->get_column_count() ) . '" class="plugin-update colspanchange"><div class="update-message">';

			$changelog_link = self_admin_url( 'index.php?edd_sl_action=view_plugin_changelog&plugin=' . $this->name . '&slug=' . $this->slug . '&TB_iframe=true&width=772&height=911' );

			if ( empty( $version_info->download_link ) ) {
				printf(
					wp_kses( /* translators: %1$s - plugin name, %2$s - changelog link, %3$s - new version number. */
						__( 'There is a new version of %1$s available. <a href="%2$s" target="_blank" class="thickbox">View version %3$s details</a>.', 'pdf-embedder-premium' ),
						[
							'a' => [
								'href'   => [],
								'class'  => [],
								'target' => [],
							],
						]
					),
					esc_html( $version_info->name ),
					esc_url( $changelog_link ),
					esc_html( $version_info->new_version )
				);
			} else {
				printf(
					wp_kses( /* translators: %1$s - plugin name, %2$s - changelog link, %3$s - new version number, %4$s - URL to initiate the update. */
						__( 'There is a new version of %1$s available. <a href="%2$s" target="_blank" class="thickbox">View version %3$s details</a> or <a href="%4$s">update now</a>.', 'pdf-embedder-premium' ),
						[
							'a' => [
								'href'   => [],
								'class'  => [],
								'target' => [],
							],
						]
					),
					esc_html( $version_info->name ),
					esc_url( $changelog_link ),
					esc_html( $version_info->new_version ),
					esc_url( wp_nonce_url( self_admin_url( 'update.php?action=upgrade-plugin&plugin=' ) . $this->name, 'upgrade-plugin_' . $this->name ) )
				);
			}

			echo '</div></td></tr>';
		}
	}

	/**
	 * Save the license status data into DB.
	 *
	 * @since 5.2.0
	 *
	 * @param stdClass $api_response What we received from the API for activate_license and get_version requests.
	 */
	protected function update_license_status_option( $api_response ): array { // phpcs:ignore Generic.Metrics.CyclomaticComplexity.MaxExceeded

		if ( $api_response === false || ( is_null( $api_response ) && ! empty( $this->license_key ) ) ) {
			// Probably unable to reach server this time.
			return [];
		}

		$license_status = [ 'license_id' => $this->license_key ];

		// Problem such as missing license key?
		if ( empty( $this->license_key ) && is_null( $api_response ) ) {
			$license_status['status'] = 'empty';
		} elseif ( isset( $api_response->license_check ) ) { // Probably called get_version.
			$license_status['status'] = $api_response->license_check;
		} elseif ( isset( $api_response->error ) ) { // activate_license.
			$license_status['status'] = $api_response->error;
		} elseif ( isset( $api_response->license ) ) {
			$license_status['status'] = $api_response->license; // check_license.
		} else { // Call was activate_license or check_license.
			$license_status['status'] = isset( $api_response->error ) ? $api_response->error :
				( isset( $api_response->success ) && $api_response->success ? 'valid' : 'invalid' );
		}

		if ( ! in_array( $license_status['status'], [ 'valid', 'invalid', 'missing', 'expired', 'site_inactive', 'inactive', 'disabled', 'empty', 'revoked', 'no_activations_left' ], true ) ) {
			$license_status['status'] = 'invalid';
		}

		$license_status['expires']       = null;
		$license_status['expires_time']  = null;
		$license_status['renewal_link']  = null;
		$license_status['download_link'] = null;

		if ( isset( $api_response->expires ) ) {
			$expires_time = strtotime( $api_response->expires );

			if ( $expires_time ) {
				$license_status['expires']      = $api_response->expires;
				$license_status['expires_time'] = $expires_time;
			}
		}
		if ( isset( $api_response->renewal_link ) ) {
			$license_status['renewal_link'] = $api_response->renewal_link;
		}
		if ( isset( $api_response->download_link ) ) {
			$license_status['download_link'] = $api_response->download_link;
		}

		// Compare to existing option if any.
		$license_status['last_check_time'] = $license_status['first_check_time'] = time();
		$license_status['result_cleared']  = false;
		$old_license_status                = get_site_option( License::OPTION_STATUS, true );

		if (
			is_array( $old_license_status ) &&
			isset( $old_license_status['license_id'] ) &&
			isset( $old_license_status['status'] )
		) {
			if (
				$old_license_status['license_id'] === $license_status['license_id'] &&
				$old_license_status['status'] === $license_status['status'] &&
				( isset( $old_license_status['expires'] ) ? $old_license_status['expires'] : 0 ) === $license_status['expires']
			) {
				if ( isset( $old_license_status['first_check_time'] ) ) {
					$license_status['first_check_time'] = $old_license_status['first_check_time'];
				}
				if ( isset( $old_license_status['result_cleared'] ) ) {
					$license_status['result_cleared'] = $old_license_status['result_cleared'];
				}
			}
		}

		update_site_option( License::OPTION_STATUS, $license_status );
		$this->license_status_option = $license_status;

		return $license_status;
	}

	/**
	 * Retrieve information about the status of the license from DB.
	 *
	 * @since 5.2.0
	 */
	protected function get_license_status_option(): array {

		if ( ! is_null( $this->license_status_option ) ) {
			return $this->license_status_option;
		}

		$this->license_status_option = get_site_option( License::OPTION_STATUS, [] );

		return $this->license_status_option;
	}

	/**
	 * Updates information on the "View version x.x details" page with custom data.
	 *
	 * @since 5.2.0
	 *
	 * @uses api_request()
	 *
	 * @param stdClass $_data   Data to be displayed to a user.
	 * @param string   $_action What is the request about.
	 * @param object   $_args   Arguments passed with the request.
	 */
	public function plugins_api_filter( $_data, $_action = '', $_args = null ) { // phpcs:ignore Generic.Metrics.CyclomaticComplexity.TooHigh

		if ( $_action !== 'plugin_information' ) {
			return $_data;
		}

		if ( ! isset( $_args->slug ) || ( $_args->slug !== $this->slug ) ) {
			return $_data;
		}

		$to_send = [
			'slug'   => $this->slug,
			'is_ssl' => is_ssl(),
			'fields' => [
				'banners' => false, // These will be supported soon, hopefully.
				'reviews' => false,
			],
		];

		// phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize
		$cache_key = 'edd_api_request_' . md5( serialize( $this->slug . $this->license_key . '-reg' ) );

		$edd_api_request_transient = $this->get_cached_version_info( $cache_key );

		// If we have no transient-saved value, run the API, set a fresh transient with the API value, and return that value too right now.
		if ( empty( $edd_api_request_transient ) ) {

			$api_response = $this->api_request( 'get_version', $to_send );  // 'plugin_information'.

			// Expires in 3 hours.
			$this->set_version_info_cache( $api_response, $cache_key );

			if ( $api_response !== false ) {
				$_data = $api_response;
			}
		} else {
			// Convert sections into an assoc array.
			if ( isset( $edd_api_request_transient->sections ) ) {
				$new_sections = [];

				foreach ( $edd_api_request_transient->sections as $k => $v ) {
					$new_sections[ $k ] = $v;
				}
				$edd_api_request_transient->sections = $new_sections;
			}

			$_data = $edd_api_request_transient;
		}

		return $_data;
	}


	/**
	 * Disable SSL verification in order to prevent download update failures.
	 *
	 * @since 5.2.0
	 *
	 * @param array  $args Arguments for the HTTP request.
	 * @param string $url  Where the request is sent.
	 */
	public function http_request_args( $args, $url ): array {

		// If it is an HTTPS request and we are performing a package download, disable SSL verification.
		if ( strpos( $url, 'https://' ) !== false && strpos( $url, 'edd-sl/package_download' ) ) {
			$args['sslverify'] = false;
		}

		return $args;
	}

	/**
	 * Call the API and, if successful, return the object delivered by the API.
	 *
	 * @since 5.2.0
	 *
	 * @param string $_action The requested action.
	 * @param array  $_data   Parameters for the API action.
	 *
	 * @return mixed False on failure, otherwise an object.
	 */
	private function api_request( $_action, $_data ) { // phpcs:ignore Generic.Metrics.CyclomaticComplexity.TooHigh

		$data = array_merge( $this->api_data, $_data );

		if ( $data['slug'] !== $this->slug ) {
			return false;
		}

		if ( empty( $this->license_key ) ) {
			return false;
		}

		if ( home_url() === self::API_URL ) {
			return false; // Don't allow a plugin to ping itself.
		}

		$api_params = [
			'edd_action' => $_action,
			'license'    => $this->license_key,
			'slug'       => $data['slug'],
			'url'        => home_url(),
		];

		// '?XDEBUG_SESSION_START=7777'
		$request = wp_remote_post(
			self::API_URL,
			[
				'timeout'   => 10,
				'sslverify' => false,
				'body'      => $api_params,
			]
		);

		if ( ! is_wp_error( $request ) ) {
			$request = json_decode( wp_remote_retrieve_body( $request ) );
		}

		if ( is_object( $request ) ) {
			$request->sections = [];

			if ( isset( $request->sections ) ) {
				$request->sections = maybe_unserialize( $request->sections );
			}

			$request->last_updated = null; // For changelog page only to avoid an error.
		}

		return $request;
	}

	/**
	 * Parse and display the changelog.
	 *
	 * @since 5.2.0
	 */
	public function show_changelog() { // phpcs:ignore Generic.Metrics.CyclomaticComplexity.MaxExceeded

		// phpcs:disable WordPress.Security.NonceVerification.Recommended
		if ( empty( $_REQUEST['edd_sl_action'] ) || $_REQUEST['edd_sl_action'] !== 'view_plugin_changelog' ) {
			return;
		}

		if ( empty( $_REQUEST['plugin'] ) ) {
			return;
		}

		if ( empty( $_REQUEST['slug'] ) ) {
			return;
		}
		// phpcs:enable WordPress.Security.NonceVerification.Recommended

		if ( ! current_user_can( 'update_plugins' ) ) {
			/** @noinspection ForgottenDebugOutputInspection */
			wp_die(
				esc_html__( 'You do not have permission to install plugin updates.', 'pdf-embedder-premium' ),
				403
			);
		}

		$cache_key    = md5( 'edd_plugin_' . sanitize_key( $_REQUEST['plugin'] ) . '_version_info-reg' );
		$version_info = $this->get_cached_version_info( $cache_key );

		if ( $version_info === false ) {

			$api_params = [
				'edd_action' => 'get_version',
				'slug'       => sanitize_text_field( wp_unslash( $_REQUEST['slug'] ) ),
				'url'        => home_url(),
			];

			$request = wp_remote_post(
				self::API_URL,
				[
					'timeout'   => 10,
					'sslverify' => false,
					'body'      => $api_params,
				]
			);

			if ( ! is_wp_error( $request ) ) {
				$version_info = json_decode( wp_remote_retrieve_body( $request ) );
			}

			if ( ! empty( $version_info ) && isset( $version_info->sections ) ) {
				$version_info->sections = maybe_unserialize( $version_info->sections );
			} else {
				$version_info = false;
			}

			if ( ! empty( $version_info ) ) {
				foreach ( $version_info->sections as $key => $section ) {
					$version_info->$key = (array) $section;
				}
			}

			$this->set_version_info_cache( $version_info, $cache_key );
		}

		if ( ! empty( $version_info ) && isset( $version_info->sections['changelog'] ) ) {
			echo '<div style="background:#fff;padding:10px;">' . esc_html( $version_info->sections['changelog'] ) . '</div>';
		}

		exit;
	}

	/**
	 * License activation logic.
	 *
	 * @since 5.2.0
	 */
	public function edd_license_activate(): array {

		// Data to send in our API request.
		$response = wp_remote_post(
			self::API_URL,
			[
				'timeout'   => 10,
				'sslverify' => false,
				'body'      => [
					'edd_action'  => 'activate_license',
					'license'     => $this->license_key,
					'url'         => trailingslashit( home_url() ),
					'environment' => function_exists( 'wp_get_environment_type' ) ? wp_get_environment_type() : 'production',
				],
			]
		);

		// Make sure the response came back okay.
		if ( wp_remote_retrieve_response_code( $response ) !== 200 || is_wp_error( $response ) ) {
			return [];
		}

		// Decode the license data.
		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		return $this->update_license_status_option( $license_data );
	}

	/**
	 * Display a notice in admin area when license is not valid.
	 *
	 * @since 5.2.0
	 */
	public function eddsl_license_notice() { // phpcs:ignore Generic.Metrics

		$license_status = $this->get_license_status_option();
		$msg            = '';

		$yes_link = pdf_embedder()->admin()->get_settings_url( 'license' );
		$yes_text = __( 'Enter License', 'pdf-embedder-premium' );

		$yes_target = '';

		if (
			isset( $license_status['status'] ) &&
			$license_status['status'] !== 'valid' &&
			( ! isset( $license_status['result_cleared'] ) || ! $license_status['result_cleared'] )
		) {
			// Wait a couple of days before warning about the issue - give them time to finish setup first.
			if ( ! isset( $license_status['first_check_time'] ) || $license_status['first_check_time'] < time() - $this->license_warning_delay ) {

				if ( $license_status['license_id'] === '' ) {
					$license_status['status'] = 'empty';
				}

				switch ( $license_status['status'] ) {
					case 'missing':
						$msg = License::get_error_text( 'missing' );
						break;

					case 'expired':
						$msg = License::get_error_text( 'expired' );

						if ( isset( $license_status['renewal_link'] ) && $license_status['renewal_link'] ) {
							$yes_link = $license_status['renewal_link'];
							$yes_text = __( 'Renew License', 'pdf-embedder-premium' );

							$yes_target = ' target="_blank" ';
						}
						break;

					case 'site_inactive':
						$msg = License::get_error_text( 'site_inactive' );
						break;

					case 'inactive':
						$msg = License::get_error_text( 'inactive' );
						break;

					case 'disabled':
						$msg = License::get_error_text( 'disabled' );
						break;

					case 'empty':
						$msg = License::get_error_text( 'empty' );
						break;

					case 'invalid':
					default:
						$msg = License::get_error_text( 'invalid' );
						break;
				}
			}
		} elseif (
			isset( $license_status['status'] ) &&
			$license_status['status'] === 'valid' &&
			( ! isset( $license_status['result_cleared'] ) || ! $license_status['result_cleared'] )
		) {
			// License valid, but will it expire soon?
			if ( isset( $license_status['expires_time'] ) && $license_status['expires_time'] < time() + MONTH_IN_SECONDS ) {
				$msg = __( 'PDF Embedder Premium license will expire soon.', 'pdf-embedder-premium' );

				if ( isset( $license_status['renewal_link'] ) && $license_status['renewal_link'] ) {
					$yes_link   = $license_status['renewal_link'];
					$yes_text   = __( 'Renew License', 'pdf-embedder-premium' );
					$yes_target = ' target="_blank" ';
				}
			}
		}

		if ( $msg !== '' ) {
			$nothanks_url = add_query_arg( License::OPTION_STATUS . '_eddsl_action', 'no_thanks' );

			echo '<div class="error"><p>';
			echo wp_kses_post( htmlentities( $msg ) );
			if ( $yes_link ) {
				echo ' &nbsp; <a href="' . esc_url( $yes_link ) . '" class="button-secondary" ' . esc_attr( $yes_target ) . '>' . wp_kses_post( htmlentities( $yes_text ) ) . '</a>';
			}
			echo '&nbsp;<a href="' . esc_url( $nothanks_url ) . '" class="button-secondary">' . esc_html__( 'Ignore', 'pdf-embedder-premium' ) . '</a>';
			echo '</p></div>';
		}
	}

	/**
	 * Get the cached version.
	 *
	 * @since 5.2.0
	 *
	 * @param string $cache_key Cache key.
	 *
	 * @return mixed False when cache expired, otherwise - an array.
	 */
	private function get_cached_version_info( string $cache_key = '' ) {

		if ( empty( $cache_key ) ) {
			$cache_key = $this->cache_key;
		}

		$cache = get_option( $cache_key );

		if ( empty( $cache['timeout'] ) || current_time( 'timestamp' ) > $cache['timeout'] ) { // @codingStandardsIgnoreLine
			return false; // Cache is expired.
		}

		return json_decode( $cache['value'] );
	}

	/**
	 * Add the plugin version information to the database.
	 *
	 * @since 5.2.0
	 *
	 * @param string $value     What to store in cache.
	 * @param string $cache_key Where to store.
	 */
	private function set_version_info_cache( $value = '', $cache_key = '' ) {

		if ( empty( $cache_key ) ) {
			$cache_key = $this->cache_key;
		}

		$data = [
			// phpcs:ignore WordPress.DateTime.CurrentTimeTimestamp.Requested
			'timeout' => strtotime( '+3 hours', current_time( 'timestamp' ) ),
			'value'   => wp_json_encode( $value ),
		];

		update_option( $cache_key, $data, false );
	}
}
