<?php

namespace PDFEmbedder\Premium\Plans\Features;

use PDFEmbedder\Options;
use PDFEmbedder\Admin\License;

/**
 * Trait ExtendsOptionsClass.
 *
 * @since 5.2.0
 */
trait ExtendsOptionsWithExtraMobile {

	/**
	 * Add the Premium options to the list of default values.
	 *
	 * @since 5.2.0
	 *
	 * @param array $defaults The default options.
	 */
	public function add_options_defaults( array $defaults ): array {

		return array_merge(
			$defaults,
			[
				'pdfemb_scrollbar'               => 'none',
				'pdfemb_continousscroll'         => 'on',
				'pdfemb_download'                => 'off',
				'pdfemb_tracking'                => 'on',
				'pdfemb_newwindow'               => 'on',
				'pdfemb_scrolltotop'             => 'off',
				'pdfemb_search'                  => 'off',
				'pdfemb_mobilewidth'             => '500',
				'pdfemb_mobilewidth_button_text' => __( 'View In Full Screen', 'pdf-embedder-premium' ),
				'pdfemb_resetviewport'           => false,
				'pdfemb_license_key'             => '',
			]
		);
	}

	/**
	 * Validate the Premium options.
	 * The validation is done in steps according to each plan priority.
	 *
	 * @since 5.2.0
	 *
	 * @param array $validated Validated options.
	 * @param array $input     Original options coming from the request.
	 */
	public function validate_options( array $validated, array $input ): array { // phpcs:ignore Generic.Metrics.CyclomaticComplexity.MaxExceeded,Generic.Metrics.NestingLevel.MaxExceeded

		$validated['pdfemb_scrollbar']       = isset( $input['pdfemb_scrollbar'] ) && in_array( $input['pdfemb_scrollbar'], [ 'vertical', 'horizontal', 'both', 'none' ], true ) ? $input['pdfemb_scrollbar'] : 'none';
		$validated['pdfemb_continousscroll'] = isset( $input['pdfemb_continousscroll'] ) && Options::is_on( $input['pdfemb_continousscroll'] ) ? 'on' : 'off';
		$validated['pdfemb_sidebar']         = isset( $input['pdfemb_sidebar'] ) && Options::is_on( $input['pdfemb_sidebar'] ) ? 'on' : 'off';
		$validated['pdfemb_download']        = isset( $input['pdfemb_download'] ) && Options::is_on( $input['pdfemb_download'] ) ? 'on' : 'off';
		$validated['pdfemb_tracking']        = isset( $input['pdfemb_tracking'] ) && Options::is_on( $input['pdfemb_tracking'] ) ? 'on' : 'off';
		$validated['pdfemb_newwindow']       = isset( $input['pdfemb_newwindow'] ) && Options::is_on( $input['pdfemb_newwindow'] ) ? 'on' : 'off';
		$validated['pdfemb_scrolltotop']     = isset( $input['pdfemb_scrolltotop'] ) && Options::is_on( $input['pdfemb_scrolltotop'] ) ? 'on' : 'off';
		$validated['pdfemb_search']          = isset( $input['pdfemb_search'] ) && Options::is_on( $input['pdfemb_search'] ) ? 'on' : 'off';

		$validated['pdfemb_download_view_counter'] = isset( $input['pdfemb_download_view_counter'] ) && Options::is_on( $input['pdfemb_download_view_counter'] ) ? 'on' : 'off';
		$validated['pdfemb_view_counter']          = isset( $input['pdfemb_view_counter'] ) && Options::is_on( $input['pdfemb_view_counter'] ) ? 'on' : 'off';
		$validated['pdfemb_download_counter']      = isset( $input['pdfemb_download_counter'] ) && Options::is_on( $input['pdfemb_download_counter'] ) ? 'on' : 'off';

		// Mobile tab.
		$validated['pdfemb_mobilewidth']             = isset( $input['pdfemb_mobilewidth'] ) ? (int) $input['pdfemb_mobilewidth'] : 500;
		$validated['pdfemb_mobilewidth_button_text'] = isset( $input['pdfemb_mobilewidth_button_text'] ) ? esc_html( $input['pdfemb_mobilewidth_button_text'] ) : false;
		$validated['pdfemb_resetviewport']           = isset( $input['pdfemb_resetviewport'] ) && Options::is_on( $input['pdfemb_resetviewport'] );

		// License Key.
		$validated['pdfemb_license_key'] = sanitize_key( $input['pdfemb_license_key'] ?? '' );

		if ( ! empty( $validated['pdfemb_license_key'] ) ) {
			if ( ! preg_match( '/^.{32}.*$/i', $validated['pdfemb_license_key'] ) ) {
				add_settings_error(
					'pdfemb_license_key',
					'too_short',
					License::get_error_text( 'too_short' ),
					'error'
				);
			} else {
				// There is a valid-looking license key present.
				$checked_license_status = get_site_option( License::OPTION_STATUS, true );

				// Only bother trying to activate if we have a new license key OR the same license key but it was invalid on last check.
				$existing_valid_license = '';

				if (
					is_array( $checked_license_status ) &&
					isset( $checked_license_status['license_id'] ) &&
					$checked_license_status['license_id'] !== '' &&
					isset( $checked_license_status['status'] ) &&
					$checked_license_status['status'] === 'valid'
				) {
					$existing_valid_license = $checked_license_status['license_id'];
				}

				if ( $existing_valid_license !== $validated['pdfemb_license_key'] ) {
					$license_status = License::activate( $validated['pdfemb_license_key'] );

					if ( isset( $license_status['status'] ) && $license_status['status'] !== 'valid' ) {
						add_settings_error(
							'pdfemb_license_key',
							$license_status['status'],
							License::get_error_text( $license_status['status'] ),
							'error'
						);
					}
				}
			}
		}

		return $validated;
	}
}
