<?php

namespace PDFEmbedder\Premium\Plans;

use PDFEmbedder\Premium\Plans\Pro\Pro;
use PDFEmbedder\Premium\Plans\Plus\Plus;
use PDFEmbedder\Premium\Plans\Elite\Elite;
use PDFEmbedder\Premium\Plans\Basic\Basic;

/**
 * Plans loader.
 *
 * @since 5.2.0
 */
final class Plans {

	/**
	 * All the loaded plans.
	 *
	 * @since 5.2.0
	 *
	 * @var PremiumPlan[] $plans
	 */
	private $plans = [];

	/**
	 * The currently active plan.
	 *
	 * @since 5.2.0
	 *
	 * @var PremiumPlan $active_plan
	 */
	private $active_plan;

	/**
	 * Initialize all the plans.
	 *
	 * @since 5.2.0
	 */
	public function init() {

		$plans = $this->register_plans();

		// Iterate over all available plans and load them.
		foreach ( $plans as $plan_class ) {
			/** @var PremiumPlan $plan */
			$plan = new $plan_class();

			$plan->init();
			$plan->hooks();

			$this->plans[ $plan::PRIORITY ] = $plan;
		}
	}

	/**
	 * Identify available plans.
	 *
	 * @since 5.2.0
	 */
	private function register_plans(): array {

		foreach ( scandir( __DIR__ . '/Available' ) as $plan_dir ) {
			if ( $plan_dir === '.' || $plan_dir === '..' ) {
				continue;
			}

			$plan_path = __DIR__ . '/Available/' . $plan_dir;

			if ( ! is_dir( $plan_path ) ) {
				continue;
			}

			$plan_class_name = __NAMESPACE__ . '\\Available\\' . $plan_dir . '\\' . $plan_dir;

			$plans[ $plan_class_name::PRIORITY ] = $plan_class_name;
		}

		// Sort the plans by priority, lowest to highest.
		ksort( $plans, SORT_NUMERIC );

		return $plans;
	}

	/**
	 * Get the currently active plan.
	 *
	 * @since 5.2.0
	 */
	public function get_active_plan(): PremiumPlan {

		$this->active_plan = end( $this->plans );

		reset( $this->plans );

		return $this->active_plan;
	}

	/**
	 * Get all plans.
	 *
	 * @since 5.2.0
	 */
	public function get(): array {

		return $this->plans;
	}
}
