<?php

namespace PDFEmbedder\Premium;

use PDFEmbedder\Admin\License;
use PDFEmbedder\Helpers\Check;
use PDFEmbedder\Plugin as LitePlugin;
use PDFEmbedder\Premium\Admin\Updater;
use PDFEmbedder\Premium\Viewer\Viewer;

/**
 * Main plugin class.
 *
 * @since 5.2.0
 */
final class Plugin {

	/**
	 * All the plans.
	 *
	 * @since 5.2.0
	 *
	 * @var Plans\Plans
	 */
	private $plans;

	/**
	 * The PFD Viewer.
	 *
	 * @since 5.2.0
	 *
	 * @var Viewer
	 */
	public $viewer;

	/**
	 * Plugin initialization logic.
	 *
	 * @since 5.2.0
	 */
	public function init() {

		if ( wp_doing_cron() || Check::is_heartbeat() ) {
			return;
		}

		$this->load_plans();
		$this->load_viewer();
	}

	/**
	 * Load all the plans.
	 *
	 * @since 5.2.0
	 */
	private function load_plans() {

		$this->plans = new Plans\Plans();

		$this->plans->init();
	}

	/**
	 * Load all the plans.
	 *
	 * @since 5.2.0
	 */
	private function load_viewer() {

		$this->viewer = new Viewer();

		$this->viewer->hooks();
	}

	/**
	 * Get access to the Plans object.
	 *
	 * @since 5.2.0
	 */
	public function plans(): Plans\Plans {

		return $this->plans;
	}

	/**
	 * Get access to the PDFViewer object.
	 *
	 * @since 5.2.0
	 */
	public function viewer(): Viewer {

		return $this->viewer;
	}

	/**
	 * Assign all hooks to proper places.
	 *
	 * @since 5.2.0
	 *
	 * @param LitePlugin $lite Instance of PDF Embedder Lite.
	 */
	public function hooks( LitePlugin $lite ) {

		$lite->premium = $this;

		add_action(
			'pdfemb_admin_init',
			static function () {

				( new Admin\Admin() )->hooks();
				( new Updater() )->hooks();
			}
		);

		add_filter(
			'pdfemb_shortcode_viewer',
            function () {
				return $this->viewer();
            }
		);

		add_filter( 'pdfemb_usage_tracking_enabled', '__return_true', PHP_INT_MAX );
		add_filter(
			'pdfemb_options_get',
			static function ( $options ) {

				$options['usagetracking'] = 'on';

				return $options;
			},
			PHP_INT_MAX
		);

		add_filter(
			'pdfemb_license_type',
			static function () {

				return pdf_embedder()->premium()->plans()->get_active_plan()::NAME;
			}
		);

		// Extend the License class with new premium-specific methods.
		License::macro(
			'activate',
			static function ( $license_key ) {
				// Set up the updater.
				return ( new Updater( $license_key ) )->edd_license_activate();
			}
		);

		add_filter(
			'pdfemb_license_status',
			static function () {
				$status = get_site_option( License::OPTION_STATUS, [] );

				return $status['status'] ?? 'lite';
			}
		);
	}
}
