<?php

namespace PDFEmbedder\Premium\Viewer;

use PDFEmbedder\Options;
use PDFEmbedder\Helpers\PdfFile;

/**
 * Class FileAction to handle various file-specific actions.
 *
 * @since 5.2.0
 */
class FileAction {

	/**
	 * The URL of the PDF file.
	 *
	 * @since 5.2.0
	 *
	 * @var string
	 */
	protected $file_url;

	/**
	 * The ID of the PDF file.
	 *
	 * @since 5.2.0
	 *
	 * @var int
	 */
	protected $file_id;

	/**
	 * FileAction constructor.
	 *
	 * @since 5.2.0
	 *
	 * @param string $file_url The URL of the PDF file.
	 * @param int    $file_id  The ID of the PDF file.
	 */
	public function __construct( string $file_url = '', int $file_id = 0 ) {

		$this->file_url = $file_url;
		$this->file_id  = absint( $file_id );
	}

	/**
	 * Increment the view count for the file.
	 *
	 * @since 5.2.0
	 */
	public function increment_view() {

		if ( ! Options::is_on( pdf_embedder()->options()->get()['pdfemb_tracking'] ) ) {
			return;
		}

		$this->guess_file_id();

		// We can't increment the view count if we don't have the file ID
		// which might be the case when the file is not hosted by WordPress.
		if ( empty( $this->file_id ) ) {
			return;
		}

		$this->do_increment( 'pdfemb-views' );
	}

	/**
	 * Increment the download count for the file.
	 *
	 * @since 5.2.0
	 */
	public function increment_download() {

		if ( ! Options::is_on( pdf_embedder()->options()->get()['pdfemb_tracking'] ) ) {
			return;
		}

		$this->guess_file_id();

		// We can't increment the download count if we don't have the file ID
		// which might be the case when the file is not hosted by WordPress.
		if ( empty( $this->file_id ) ) {
			return;
		}

		$this->do_increment( 'pdfemb-downloads' );
	}

	/**
	 * Increment the view or download count for the file.
	 *
	 * @since 5.2.0
	 *
	 * @param string $entity The entity to increment.
	 */
	private function do_increment( string $entity ) {

		if ( ! ( $entity === 'pdfemb-views' || $entity === 'pdfemb-downloads' ) ) {
			return;
		}

		global $wpdb;

		// phpcs:ignore WordPress.DB.DirectDatabaseQuery.NoCaching
		$meta_id = $wpdb->get_var(
			$wpdb->prepare(
				"SELECT meta_id FROM $wpdb->postmeta
				WHERE post_id = %d AND meta_key = %s;",
				$this->file_id,
				$entity
			)
		);

		if ( empty( $meta_id ) ) {
			// phpcs:ignore WordPress.DB.DirectDatabaseQuery.NoCaching
			$wpdb->query(
				$wpdb->prepare(
					"INSERT INTO $wpdb->postmeta (post_id, meta_key, meta_value)
					VALUES (%d, %s, 1)",
					$this->file_id,
					$entity
				)
			);
		} else {
			// phpcs:ignore WordPress.DB.DirectDatabaseQuery.NoCaching
			$wpdb->query(
				$wpdb->prepare(
					"UPDATE $wpdb->postmeta SET meta_value = meta_value + 1
	                WHERE post_id = %d AND meta_key = %s;",
					$this->file_id,
					$entity
				)
			);
		}
	}

	/**
	 * Guess the file ID by its URL.
	 *
	 * @since 5.2.0
	 */
	protected function guess_file_id() {

		if ( empty( $this->file_url ) ) {
			return;
		}

		$file_id_by_url = PdfFile::get_id_by_url( $this->file_url );

		if ( $this->file_id === 0 ) {
			// This is the case when URL was not supplied,
			// or when the file ID was not provided in the constructor.
			$this->file_id = $file_id_by_url;
		} elseif ( $this->file_id !== $file_id_by_url ) {
			// Do not allow to pass a wrong file ID that is not related to the provided URL.
			// They must match, otherwise no actions will be performed.
			$this->file_id = 0;
		}
	}
}
