<?php

namespace PDFEmbedder\Premium\Viewer;

use PDFEmbedder\Options;
use PDFEmbedder\Helpers\PdfFile;
use PDFEmbedder\Premium\Helpers\Assets;
use PDFEmbedder\Viewer\ViewerInterface;

/**
 * Main PDF renderer.
 *
 * @since 5.2.0
 */
class Viewer implements ViewerInterface {

	/**
	 * The version of the currently used PDFJS.
	 *
	 * @since 5.2.0
	 */
	const PDFJS_VERSION = '2.16.105';

	/**
	 * Processed and ready-to-use attributes.
	 *
	 * @since 5.2.0
	 *
	 * @var array
	 */
	protected $atts = [];

	/**
	 * All plugin options.
	 *
	 * @since 5.2.0
	 *
	 * @var array
	 */
	protected $options;

	/**
	 * Pass the options needed for the viewer.
	 *
	 * @since 5.2.0
	 *
	 * @param array $atts Attributes.
	 */
	public function set_options( array $atts = [] ) {

		$this->options = pdf_embedder()->options()->get();

		$this->atts = $atts;

		if ( empty( $this->atts['url'] ) && ! empty( $this->atts['pdfemb-serveurl'] ) ) {
			$this->atts['url'] = $this->atts['pdfemb-serveurl'];
		}
	}

	/**
	 * Let's set up the viewer.
	 *
	 * @since 5.2.0
	 */
	public function hooks() {

		// Set the prior.
		add_filter( 'template_include', [ $this, 'template_include' ], 20001 );
		add_filter( 'show_admin_bar', [ $this, 'hide_viewer_admin_bar' ] );

		// Allow scripts to be injected into the viewer iframe
		// without calling wp_head/wp_footer actions to prevent code injections.
		add_action( 'wp_pdf_viewer_head', 'wp_enqueue_scripts', 1 );
		add_action( 'wp_pdf_viewer_head', 'wp_preload_resources', 1 );
		add_action( 'wp_pdf_viewer_head', 'wp_print_styles', 1 );
		add_action( 'wp_pdf_viewer_head', 'wp_print_head_scripts', 1 );
		add_action( 'wp_pdf_viewer_head', 'wp_custom_css_cb', 1 );
		add_action( 'wp_pdf_viewer_head', 'wp_site_icon', 99 );
		add_action( 'wp_pdf_viewer_head', 'wp_maybe_inline_styles', 1 );
		add_action( 'wp_pdf_viewer_head', 'wp_print_scripts', 20 );
		add_action( 'wp_pdf_viewer_head', [ $this, 'render_viewer_head_meta_info' ] );
		add_action( 'wp_pdf_viewer_head', [ $this, 'render_mobile_viewport' ] );

		add_action( 'wp_pdf_viewer_footer', 'wp_print_footer_scripts',1 );
		add_action( 'wp_pdf_viewer_footer', 'wp_enqueue_global_styles',1 );
		add_action( 'wp_pdf_viewer_footer', 'wp_enqueue_stored_styles', 1 );
		add_action( 'wp_pdf_viewer_footer', 'wp_maybe_inline_styles', 1 );

		// Make sure theme scripts/styles don't load on the viewer.php.
		add_action( 'wp_print_styles', [ $this, 'remove_styles_viewer' ], 99999 );
		add_action( 'wp_print_scripts', [ $this, 'remove_scripts_viewer' ], 99999 );

		// Tracking functionality.
		add_action( 'wp_ajax_pdfemb_tracking', [ $this, 'record_tracking' ] );
		add_action( 'wp_ajax_no_priv_pdfemb_tracking', [ $this, 'record_tracking' ] );
	}

	/**
	 * In the viewer add a canonical link to PDF file for public and robots[noindex] for secured PDF.
	 *
	 * @since 5.2.0
	 */
	public function render_viewer_head_meta_info() {

		$is_secure = ! strpos( $this->atts['url'], '/securepdfs/' ) === false;

		if ( $is_secure ) {
			echo '<meta name="robots" content="noindex, nofollow" />';

			return;
		}
		?>

		<link rel="canonical" href="<?php echo esc_url( $this->atts['url'] ); ?>" />
		<meta name="robots" content="index, nofollow" />

		<?php
	}

	/**
	 * Track some file-related actions.
	 *
	 * @since 5.2.0
	 */
	public function record_tracking() { // phpcs:ignore Generic.Metrics.CyclomaticComplexity.TooHigh

		$method = sanitize_key( $_POST['method'] ?? '' );

		if ( ! in_array( $method, [ 'view', 'download' ], true ) ) {
			wp_send_json_error();
		}

		$nonce   = sanitize_key( $_POST['nonce'] ?? '' );
		$pdf_url = sanitize_url( wp_unslash( $_POST['url'] ?? '' ) );

		if ( empty( $pdf_url ) || empty( $nonce ) ) {
			wp_send_json_error();
		}

		if ( ! wp_verify_nonce( $nonce, 'pdfemb-tracking-nonce-' . $pdf_url ) ) {
			wp_send_json_error();
		}

		$pdf_id = absint( $_POST['id'] ?? 0 );

		switch ( $method ) {
			case 'view':
				( new FileAction( $pdf_url, $pdf_id ) )->increment_view();
				break;

			case 'download':
				( new FileAction( $pdf_url, $pdf_id ) )->increment_download();
				break;
		}

		wp_send_json_success();
	}

	/**
	 * Enqueue scripts.
	 *
	 * @since 5.2.0
	 */
	public function enqueue_inline_assets() { // phpcs:ignore Generic.Metrics.CyclomaticComplexity.TooHigh

		if ( ! $this->is_viewer() ) {
			return;
		}

		wp_enqueue_script(
			'pdfemb-viewer',
			Assets::url( 'js/pdf-viewer.min.js', false ),
			[ 'jquery' ],
			self::PDFJS_VERSION
		);

		$strings = apply_filters(
			'pdfemb_premium_viewer_strings',
			[
				'ajaxurl'         => esc_url( admin_url( 'admin-ajax.php' ) ),
				'workerSrc'       => Assets::url( 'js/pdf-worker.min.js', false ),
				'cMapUrl'         => trailingslashit( PDFEMB_PREMIUM_PLUGIN_URL ) . 'assets/js/pdfjs/cmaps/',
				'objectL10n'      => [
					'loading'            => esc_attr__( 'Loading...', 'pdf-embedder-premium' ),
					'page'               => esc_attr__( 'Page', 'pdf-embedder-premium' ),
					'zoom'               => esc_attr__( 'Zoom', 'pdf-embedder-premium' ),
					'prev'               => esc_attr__( 'Previous page', 'pdf-embedder-premium' ),
					'next'               => esc_attr__( 'Next page', 'pdf-embedder-premium' ),
					'zoomin'             => esc_attr__( 'Zoom In', 'pdf-embedder-premium' ),
					'zoomout'            => esc_attr__( 'Zoom Out', 'pdf-embedder-premium' ),
					'secure'             => esc_attr__( 'Secure', 'pdf-embedder-premium' ),
					'download'           => esc_attr__( 'Download PDF', 'pdf-embedder-premium' ),
					'fullscreen'         => esc_attr__( 'Full Screen', 'pdf-embedder-premium' ),
					'domainerror'        => esc_attr__( 'Error: URL to the PDF file must be on exactly the same domain as the current web page.', 'pdf-embedder-premium' ),
					'clickhereinfo'      => esc_attr__( 'Click here for more info', 'pdf-embedder-premium' ),
					'widthheightinvalid' => esc_attr__( 'PDF page width or height are invalid', 'pdf-embedder-premium' ),
					'viewinfullscreen'   => esc_attr__( 'View in Full Screen', 'pdf-embedder-premium' ),
				],
				'tracking'        => Options::is_on( $this->atts['tracking'] ?? $this->options['pdfemb_tracking'] ),
				'trackingNonce'   => wp_create_nonce( 'pdfemb-tracking-nonce-' . $this->atts['url'] ),
				'fullscreenMode'  => sanitize_key( $this->atts['fullscreenMode'] ?? 'off' ),
				'zoom'            => sanitize_key( $this->atts['zoom'] ?? 'page-fit' ),
				'fpzoom'          => sanitize_key( $this->atts['fpzoom'] ?? 'page-fit' ),
				'newwindow'       => sanitize_key( $this->atts['newwindow'] ?? $this->options['newwindow'] ?? 'on' ),
				'continousscroll' => Options::is_on( $this->atts['continousscroll'] ?? $this->options['pdfemb_continousscroll'] ),
				'scrolltotop'     => Options::is_on( $this->atts['scrolltotop'] ?? $this->options['pdfemb_scrolltotop'] ),
				'mobilewidth'     => (int) ( $this->atts['mobilewidth'] ?? $this->options['pdfemb_mobilewidth'] ),
			],
			$this->atts
		);

		add_filter( 'pdfemb_premium_viewer_strings', static function( $atts ) {
			$atts['zoom']   = 'page-actual';
			$atts['fpzoom'] = 'page-actual';

			return $atts;
		} );

		wp_add_inline_script(
			'pdfemb-viewer',
			'const pdfemb_trans =' . wp_json_encode( $strings ),
			'before'
		);

		wp_enqueue_style( 'pdfemb-viewer', Assets::url( 'css/pdf-embedder.css' ), null, Assets::ver(), 'all' );
		wp_enqueue_style( 'pdfemb-viewer-print', Assets::url( 'css/pdf-embedder-print.css' ), null, Assets::ver(), 'print' );
	}

	/**
	 * Generate content of the iframe.
	 *
	 * @since 5.2.0
	 *
	 * @param string $template Current Template.
	 */
	public function template_include( string $template ) {

		if ( ! $this->is_viewer() ) {
			return $template;
		}

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$this->set_options( $_GET );
		$this->enqueue_inline_assets();

		$this->build_iframe_content();

		return false;
	}

	/**
	 * PDF Embed Head Helper.
	 *
	 * @since 5.2.0
	 */
	public function render_mobile_viewport() {

		if ( $this->options['pdfemb_resetviewport'] ) {
			echo '<meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1, user-scalable=0" />';
		}
	}

	/**
	 * Check if the current page is the viewer.
	 *
	 * @since 5.2.0
	 */
	public function is_viewer(): bool {

		// phpcs:disable WordPress.Security.NonceVerification.Recommended,Universal.WhiteSpace.PrecisionAlignment.Found
		return isset( $_GET['pdfID'] ) &&
		       (int) $_GET['pdfID'] >= 0 &&
		       (
				   empty( $_GET['url'] ) ||
				   empty( $_GET['pdfemb-serveurl'] )
		       );
		// phpcs:enable WordPress.Security.NonceVerification.Recommended,Universal.WhiteSpace.PrecisionAlignment.Found
	}

	/**
	 * Hide Viewer Admin Bar inside the viewer.
	 *
	 * @since 5.2.0
	 *
	 * @param bool $show Whether to show admin bar.
	 */
	public function hide_viewer_admin_bar( bool $show ): bool {

		if ( $this->is_viewer() ) {
			return false;
		}

		return $show;
	}

	/**
	 * Helper method to remove viewer styles.
	 *
	 * @since 5.2.0
	 */
	public function remove_styles_viewer() {

		global $wp_styles;

		if ( ! $this->is_viewer() ) {
			return;
		}

		// Run through the queue styles.
		foreach ( $wp_styles->queue as $handle ) {
			if (
				strpos( $handle, 'wppdfemb-' ) !== false ||
				strpos( $handle, 'pdfemb-' ) !== false
			) {
				continue;
			}

			wp_dequeue_style( $handle );
		}
	}

	/**
	 * Helper method to remove viewer scripts.
	 *
	 * @since 5.2.0
	 */
	public function remove_scripts_viewer() {

		global $wp_scripts;

		if ( ! $this->is_viewer() ) {
			return;
		}

		// Run through the queue styles.
		foreach ( $wp_scripts->queue as $handle ) {
			if (
				strpos( $handle, 'wppdfemb-' ) !== false ||
				strpos( $handle, 'pdfemb-' ) !== false
			) {
				continue;
			}

			wp_dequeue_script( $handle );
		}
	}

	/**
	 * Get the viewer template part.
	 *
	 * @since 5.2.0
	 *
	 * @param string $part Template part name.
	 */
	protected function get_template_part( string $part ): string {

		$part = sanitize_key( $part );

		// Include a template part from the plugin /templates/ directory.
		$templates_dir = PDFEMB_PREMIUM_PLUGIN_DIR . 'templates/viewer/';

		if ( file_exists( $templates_dir . $part . '.php' ) ) {
			ob_start();
			include $templates_dir . $part . '.php';

			return ob_get_clean();
		}

		return '';
	}

	/**
	 * Build the viewer HTML.
	 *
	 * @since 5.2.0
	 */
	public function build_iframe_content() { // phpcs:ignore Generic.Metrics.CyclomaticComplexity

		if ( empty( $this->atts['url'] ) && ! empty( $this->atts['pdfemb-serveurl'] ) ) {
			$this->atts['url'] = $this->atts['pdfemb-serveurl'];
		}

		// If there's no URL then no sense generating the viewer.
		if ( empty( esc_url_raw( $this->atts['url'] ) ) ) {
			return false;
		}

		// get the main options.
		$options = $this->options;

		// Special cases: not in options.
		$pdf_embed_index = isset( $this->atts['index'] ) ? (int) $this->atts['index'] : 1;

		$toolbar      = $this->atts['toolbar'] ?? $options['pdfemb_toolbar'];
		$toolbarfixed = $this->atts['toolbarfixed'] ?? $options['pdfemb_toolbarfixed'];
		$start_page   = $this->atts['page'] ?? 1;
		$scrollbar    = $this->atts['scrollbar'] ?? $options['pdfemb_scrollbar'];

		$is_search_active        = Options::is_on( $this->atts['search'] ?? $options['pdfemb_search'] );
		$is_download_btn_enabled = Options::is_on( $this->atts['download'] ?? $options['pdfemb_download'] );

		$continousscroll = Options::is_on( $this->atts['continousscroll'] ?? $options['pdfemb_continousscroll'] ) ? 'on' : 'off';

		// Mobile.
		$mobile_width       = (int) ( $this->atts['mobilewidth'] ?? $options['pdfemb_mobilewidth'] );
		$mobile_button_text = $this->atts['mobilewidth_button_text'] ?? $options['pdfemb_mobilewidth_button_text'];

		$height = '100%';
		$width  = '100%';
		?>

		<!DOCTYPE html>
		<html id="wp-pdf-embbed" style="height: 100%;" <?php language_attributes(); ?>>

			<?php
			// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			echo $this->get_template_part( 'header' );
			?>

			<body id="wppdf-iframe-body" tabindex="1">

				<div id="pdf-data"
					data-index="<?php echo esc_attr( $pdf_embed_index ); ?>"
					class="pdf-embed pdf-embed-<?php echo esc_attr( $pdf_embed_index ); ?> pdf-embed-container <?php echo ( ! empty( $mobile_width ) ? 'mobile-view-active mobile-view-' . (int) $mobile_width : '' ); ?>"
					data-file="<?php echo esc_url( $this->atts['url'] ); ?>"
					data-pdfid="<?php echo esc_attr( $this->atts['pdfID'] ); ?>"
					data-page="<?php echo (int) $start_page; ?>"
					data-toolbar="<?php echo esc_attr( $toolbar ); ?>"
					data-height="<?php echo esc_attr( $height ); ?>"
					style="height: <?php echo esc_attr( $height ); ?>;"
					<?php if ( ! empty( $mobile_width ) ) { ?>
						data-mobile-width="<?php echo (int) $mobile_width; ?>"
					<?php } ?>
				>

				<div id="outerContainer">

					<?php if ( ! empty( $mobile_button_text ) ) { ?>
						<div class="pdfemb-inner-div-wantmobile-fswrap pdfemb-wantmobile">
							<div class="pdfemb-wantmobile-fsarea"><?php echo esc_html( $mobile_button_text ); ?></div>
						</div>
					<?php } ?>

					<?php do_action( 'pdfemb_viewer_before_main_container' ); ?>

					<div id="mainContainer">

						<?php // if ( $toolbar === 'top' || $toolbar === 'both' ) : ?>

							<?php if ( $is_search_active ) : ?>
								<div class="findbar hidden doorHanger findbar-<?php echo esc_attr( $toolbar ); ?>" id="findbar">
									<div id="findbarInputContainer">
										<input id="findInput" class="findInput toolbarField" title="<?php esc_attr_e( 'Find', 'pdf-embedder-premium' ); ?>" placeholder="<?php esc_attr_e( 'Find in document…', 'pdf-embedder-premium' ); ?>" tabindex="91" data-l10n-id="find_input" aria-invalid="false">
										<div class="splitToolbarButton">
											<button id="findPrevious" class="findPrevious toolbarButton" title="<?php esc_attr_e( 'Find the previous occurrence of the phrase', 'pdf-embedder-premium' ); ?>" tabindex="92" data-l10n-id="find_previous">
												<span data-l10n-id="find_previous_label"><?php esc_html_e( 'Previous', 'pdf-embedder-premium' ); ?></span>
											</button>
											<!--<div class="splitToolbarButtonSeparator"></div>-->
											<button id="findNext" class="findNext toolbarButton" title="<?php esc_attr_e( 'Find the next occurrence of the phrase', 'pdf-embedder-premium' ); ?>" tabindex="93" data-l10n-id="find_next">
												<span data-l10n-id="find_next_label"><?php esc_html_e( 'Next', 'pdf-embedder-premium' ); ?></span>
											</button>
										</div>
									</div>

									<div id="findbarOptionsOneContainer">
										<input type="checkbox" id="findHighlightAll" class="findHighlightAll toolbarField" tabindex="94">
										<label for="findHighlightAll" class="toolbarLabel" data-l10n-id="find_highlight"><?php esc_html_e( 'Highlight All', 'pdf-embedder-premium' ); ?></label>
										<input type="checkbox" id="findMatchCase" class="findMatchCase toolbarField" tabindex="95">
										<label for="findMatchCase" class="toolbarLabel" data-l10n-id="find_match_case_label"><?php esc_html_e( 'Match Case', 'pdf-embedder-premium' ); ?></label>
									</div>
									<div id="findbarOptionsTwoContainer">
										<input type="checkbox" id="findMatchDiacritics" class="findMatchDiacritics toolbarField" tabindex="96">
										<label for="findMatchDiacritics" class="toolbarLabel" data-l10n-id="find_match_diacritics_label"><?php esc_html_e( 'Match Diacritics', 'pdf-embedder-premium' ); ?></label>
										<input type="checkbox" id="findEntireWord" class="findEntireWord toolbarField" tabindex="97">
										<label for="findEntireWord" class="toolbarLabel" data-l10n-id="find_entire_word_label"><?php esc_html_e( 'Whole Words', 'pdf-embedder-premium' ); ?></label>
									</div>

									<div id="findbarMessageContainer" class="findbarMessageContainer" aria-live="polite">
										<span id="findResultsCount" class="findResultsCount toolbarLabel"></span>
										<span id="findMsg" class="findMsg toolbarLabel"></span>
									</div>
								</div><!-- findbar -->
							<?php endif; ?>

							<?php do_action( 'pdfemb_viewer_between_findbar_toolbar' ); ?>

							<div class="toolbar pdfemb-toolbar-always-visible-<?php echo esc_attr( $toolbarfixed ); ?> pdfemb-toolbar-display-<?php echo esc_attr( $toolbar ); ?>">

								<div id="toolbarContainer" class="toolbarContainer">
									<div id="toolbarViewer" class="toolbarViewer">
										<div id="toolbarViewerLeft" class="toolbarViewerLeft">

											<?php if ( $is_search_active ) : ?>
												<button id="viewFind" class="viewFind toolbarButton findbarOn" title="<?php esc_attr_e( 'Find in Document', 'pdf-embedder-premium' ); ?>" tabindex="12" data-l10n-id="findbar" aria-expanded="false" aria-controls="findbar">
													<span data-l10n-id="findbar_label"><?php esc_html_e( 'Find', 'pdf-embedder-premium' ); ?></span>
												</button>
											<?php endif; ?>

											<div class="splitToolbarButton">
												<button class="toolbarButton previousButton" title="<?php esc_attr_e( 'Previous Page', 'pdf-embedder-premium' ); ?>" id="previous" tabindex="13" data-l10n-id="previous">
													<span data-l10n-id="previous_label"><?php esc_html_e( 'Previous', 'pdf-embedder-premium' ); ?></span>
												</button>

												<button class="toolbarButton nextButton next-bottom" title="<?php esc_attr_e( 'Next Page', 'pdf-embedder-premium' ); ?>" id="next" tabindex="14" data-l10n-id="next">
													<span data-l10n-id="next_label"><?php esc_html_e( 'Next', 'pdf-embedder-premium' ); ?></span>
												</button>
											</div>

											<span class="numPageStart"><?php esc_html_e( 'Page', 'pdf-embedder-premium' ); ?></span>
											<input type="number" id="pageNumber" class="pageNumber toolbarField" title="Page" value="1" size="4" min="1" tabindex="15" data-l10n-id="page" autocomplete="off">
											<span class="numbPagesDivider">/</span>
											<span id="numPages" class="numPages toolbarLabel"></span>
										</div>

										<div id="toolbarViewerRight" class="toolbarViewerRight">

											<?php
											$print_button_class = isset( $options['pdfemb_print'] ) && 'on' === strtolower( $options['pdfemb_print'] ) ? 'printButtonOn' : 'printButtonHidden';
											?>

											<button id="print" class="toolbarButton <?php echo esc_attr( $print_button_class ); ?>" title="<?php esc_attr_e( 'Print', 'pdf-embedder-premium' ); ?>" tabindex="33" data-l10n-id="print">
												<span data-l10n-id="print_label"><?php esc_html_e( 'Print', 'pdf-embedder-premium' ); ?></span>
											</button>

											<?php if ( $is_download_btn_enabled ) : ?>
												<button id="download" class="download toolbarButton dlButtonOn" title="Download" tabindex="34" data-l10n-id="download">
													<span data-l10n-id="download_label"><?php esc_html_e( 'Download', 'pdf-embedder-premium' ); ?></span>
												</button>
											<?php endif; ?>

											<?php
											$fs_button_class = isset( $options['pdfemb_fullscreen'] ) && 'on' === strtolower( $options['pdfemb_fullscreen'] ) ? 'fsButtonOn' : 'fsButtonHidden';
											?>

											<button id="fullscreen" data-pdf-id="<?php echo esc_attr( $this->atts['pdfID'] ); ?>" class="toolbarButton wppdf-fullscreen-button wppdf-embed-fullscreen-<?php echo esc_attr( $this->atts['pdfID'] ); ?> <?php echo esc_attr( $fs_button_class ); ?>" title="Fullscreen" tabindex="35" data-l10n-id="fullscreen" data-popup="#pdf-data">
												<span data-l10n-id="fullscreen_label"><?php esc_html_e( 'Fullscreen', 'pdf-embedder-premium' ); ?></span>
											</button>

											<!-- Should be visible when the "editorModeButtons" are visible. -->
											<div id="editorModeSeparator" class="editorModeSeparator verticalToolbarSeparator hidden"></div>

										</div>

										<div id="toolbarViewerMiddle" class="toolbarViewerMiddle">
											<div class="splitToolbarButton">
												<button id="zoomOut" class="zoomOut toolbarButton" title="<?php esc_attr_e( 'Zoom Out', 'pdf-embedder-premium' ); ?>" tabindex="21" data-l10n-id="zoom_out">
													<span data-l10n-id="zoom_out_label"><?php esc_html_e( 'Zoom Out', 'pdf-embedder-premium' ); ?></span>
												</button>
												<button id="zoomIn" class="zoomIn toolbarButton" title="<?php esc_attr_e( 'Zoom In', 'pdf-embedder-premium' ); ?>" tabindex="22" data-l10n-id="zoom_in">
													<span data-l10n-id="zoom_in_label"><?php esc_html_e( 'Zoom In', 'pdf-embedder-premium' ); ?></span>
												</button>
											</div>

											<span id="scaleSelectContainer" class="scaleSelectContainer dropdownToolbarButton">
												<select id="scaleSelect" class="scaleSelect" title="<?php esc_attr_e( 'Zoom', 'pdf-embedder-premium' ); ?>" tabindex="23" data-l10n-id="zoom">
													<option id="pageAutoOption" title="" value="auto" data-l10n-id="page_scale_auto"><?php esc_html_e( 'Automatic Zoom', 'pdf-embedder-premium' ); ?></option>
													<option id="pageActualOption" title="" value="page-actual" data-l10n-id="page_scale_actual"><?php esc_html_e( 'Actual Size', 'pdf-embedder-premium' ); ?></option>
													<option id="pageFitOption" title="" value="page-fit" selected="selected" data-l10n-id="page_scale_fit"><?php esc_html_e( 'Page Fit', 'pdf-embedder-premium' ); ?></option>
													<option id="pageWidthOption" title="" value="page-width" data-l10n-id="page_scale_width"><?php esc_html_e( 'Page Width', 'pdf-embedder-premium' ); ?></option>
													<option id="customScaleOption" title="" value="custom" disabled="disabled" hidden="true"></option>
													<option title="" value="0.5" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 50 }'><?php esc_html_e( '50%', 'pdf-embedder-premium' ); ?></option>
													<option title="" value="0.75" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 75 }'><?php esc_html_e( '75%', 'pdf-embedder-premium' ); ?></option>
													<option title="" value="1" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 100 }'><?php esc_html_e( '100%', 'pdf-embedder-premium' ); ?></option>
													<option title="" value="1.25" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 125 }'><?php esc_html_e( '125%', 'pdf-embedder-premium' ); ?></option>
													<option title="" value="1.5" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 150 }'><?php esc_html_e( '150%', 'pdf-embedder-premium' ); ?></option>
													<option title="" value="2" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 200 }'><?php esc_html_e( '200%', 'pdf-embedder-premium' ); ?></option>
													<option title="" value="3" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 300 }'><?php esc_html_e( '300%', 'pdf-embedder-premium' ); ?></option>
													<option title="" value="4" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 400 }'><?php esc_html_e( '400%', 'pdf-embedder-premium' ); ?></option>

													<?php do_action( 'pdfemb_viewer_toolbar_scale_percent' ); ?>
												</select>
											</span>
										</div>
									</div>

									<div id="loadingBar" class="loadingBar">
										<div class="progress">
											<div class="glimmer"></div>
										</div>
									</div>
								</div>
							</div>

						<?php // endif; ?>

						<div id="viewerContainer" tabindex="0"
							class="scrollbar-<?php echo esc_attr( $scrollbar ); ?>
							continousscroll-<?php echo esc_attr( $continousscroll ); ?>
							toolbar-<?php echo esc_attr( $toolbar ); ?>">

							<div id="viewer" class="pdfViewer"></div>
						</div>

						<div id="errorWrapper" hidden="true">
							<div id="errorMessageLeft">
								<span id="errorMessage"></span>
								<button id="errorShowMore" data-l10n-id="error_more_info"><?php esc_html_e( 'More Information', 'pdf-embedder-premium' ); ?></button>
								<button id="errorShowLess" data-l10n-id="error_less_info" hidden="true"><?php esc_html_e( 'Less Information', 'pdf-embedder-premium' ); ?></button>
							</div>
							<div id="errorMessageRight">
								<button id="errorClose" data-l10n-id="error_close"><?php esc_html_e( 'Close', 'pdf-embedder-premium' ); ?></button>
							</div>
							<div id="errorSpacer"></div>
							<textarea id="errorMoreInfo" hidden="true" readonly="readonly"></textarea>
						</div>

					</div><!-- mainContainer -->

					<?php
					// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					echo $this->get_template_part( 'dialogs' );
					?>

					<div id="printContainer"></div>

					<div class="pdfmedia">
						<?php if ( $toolbar === 'bottom' || $toolbar === 'both' ) : ?>

							<?php if ( $is_search_active ) : ?>
								<div class="findbar hidden doorHanger findbar-bottom" id="findbar-bottom">
									<div id="findbarInputContainer">
										<input id="findInput-bottom" class="findInput toolbarField" title="Find" placeholder="<?php esc_attr_e( 'Find in document…', 'pdf-embedder-premium' ); ?>" tabindex="91" data-l10n-id="find_input" aria-invalid="false">
										<div class="splitToolbarButton">
											<button id="findPrevious" class="findPrevious toolbarButton" title="<?php esc_attr_e( 'Find the previous occurrence of the phrase', 'pdf-embedder-premium' ); ?>" tabindex="92" data-l10n-id="find_previous">
												<span data-l10n-id="find_previous_label"><?php esc_html_e( 'Previous', 'pdf-embedder-premium' ); ?></span>
											</button>
											<button id="findNext-bottom" class="findNext toolbarButton" title="<?php esc_attr_e( 'Find the next occurrence of the phrase', 'pdf-embedder-premium' ); ?>" tabindex="93" data-l10n-id="find_next">
												<span data-l10n-id="find_next_label"><?php esc_html_e( 'Next', 'pdf-embedder-premium' ); ?></span>
											</button>
										</div>
									</div>

									<div id="findbarOptionsOneContainer">
										<input type="checkbox" id="findHighlightAll-bottom" class="findHighlightAll toolbarField" tabindex="94">
										<label for="findHighlightAll-bottom" class="toolbarLabel" data-l10n-id="find_highlight"><?php esc_html_e( 'Highlight All', 'pdf-embedder-premium' ); ?></label>
										<input type="checkbox" id="findMatchCase-bottom" class="findMatchCase toolbarField" tabindex="95">
										<label for="findMatchCase-bottom" class="toolbarLabel" data-l10n-id="find_match_case_label"><?php esc_html_e( 'Match Case', 'pdf-embedder-premium' ); ?></label>
									</div>
									<div id="findbarOptionsTwoContainer">
										<input type="checkbox" id="findMatchDiacritics-bottom" class="findMatchDiacritics toolbarField" tabindex="96">
										<label for="findMatchDiacritics-bottom" class="toolbarLabel" data-l10n-id="find_match_diacritics_label"><?php esc_html_e( 'Match Diacritics', 'pdf-embedder-premium' ); ?></label>
										<input type="checkbox" id="findEntireWord-bottom" class="findEntireWord toolbarField" tabindex="97">
										<label for="findEntireWord" class="toolbarLabel" data-l10n-id="find_entire_word_label"><?php esc_html_e( 'Whole Words', 'pdf-embedder-premium' ); ?></label>
									</div>

									<div id="findbarMessageContainer-bottom" class="findbarMessageContainer" aria-live="polite">
										<span id="findResultsCount-bottom" class="findResultsCount toolbarLabel"></span>
										<span id="findMsg-bottom" class="findMsg toolbarLabel"></span>
									</div>
								</div>  <!-- findbar -->
							<?php endif; ?>

							<?php do_action( 'pdfemb_viewer_between_findbar_toolbar' ); ?>

							<div id="toolbar-bottom" class="toolbar pdfemb-toolbar-always-visible-<?php echo esc_attr( $toolbarfixed ); ?> pdfemb-toolbar-display-bottom">
								<div id="toolbarContainer-bottom" class="toolbarContainer">
									<div id="toolbarViewer-bottom" class="toolbarViewer">
										<div id="toolbarViewerLeft-bottom" class="toolbarViewerLeft">
											<?php if ( $is_search_active ) : ?>
												<button id="viewFind-bottom" class="viewFind viewfind-bottom toolbarButton findbarOn" title="<?php esc_attr_e( 'Find in Document', 'pdf-embedder-premium' ); ?>" tabindex="12" data-l10n-id="findbar" aria-expanded="false" aria-controls="findbar">
													<span data-l10n-id="findbar_label"><?php esc_html_e( 'Find', 'pdf-embedder-premium' ); ?></span>
												</button>
											<?php endif; ?>

											<div class="splitToolbarButton">
												<button class="toolbarButton previousButton previous-bottom" title="<?php esc_attr_e( 'Previous Page', 'pdf-embedder-premium' ); ?>" id="previous-bottom" tabindex="13" data-l10n-id="previous">
													<span data-l10n-id="previous_label"><?php esc_html_e( 'Previous', 'pdf-embedder-premium' ); ?></span>
												</button>
												<!--<div class="splitToolbarButtonSeparator"></div>-->
												<button class="toolbarButton nextButton next-bottom" title="<?php esc_attr_e( 'Next Page', 'pdf-embedder-premium' ); ?>" id="next-bottom" tabindex="14" data-l10n-id="next">
													<span data-l10n-id="next_label"><?php esc_html_e( 'Next', 'pdf-embedder-premium' ); ?></span>
												</button>
											</div>

											<span class="numPageStart"><?php esc_html_e( 'Page', 'pdf-embedder-premium' ); ?></span>
											<input type="number" id="pageNumber-bottom" class="pageNumber toolbarField" title="<?php esc_attr_e( 'Page', 'pdf-embedder-premium' ); ?>" value="1" size="4" min="1" tabindex="15" data-l10n-id="page" autocomplete="off">
											<span class="numbPagesDivider">/</span>
											<span id="numPages-bottom" class="numPages toolbarLabel"></span>
										</div>

										<div id="toolbarViewerRight" class="toolbarViewerRight">

											<?php
											$print_button_class = isset( $options['pdfemb_print'] ) && strtolower( $options['pdfemb_print'] ) === 'on' ? 'printButtonOn' : 'printButtonHidden';
											?>
											<button id="print-bottom" class="toolbarButton <?php echo esc_attr( $print_button_class ); ?>" title="<?php esc_attr_e( 'Print', 'pdf-embedder-premium' ); ?>" tabindex="33" data-l10n-id="print">
												<span data-l10n-id="print_label"><?php esc_html_e( 'Print', 'pdf-embedder-premium' ); ?></span>
											</button>

											<?php if ( $is_download_btn_enabled ) : ?>
												<button id="download-bottom" class="download toolbarButton dlButtonOn" title="<?php esc_attr_e( 'Download', 'pdf-embedder-premium' ); ?>" tabindex="34" data-l10n-id="download">
													<span data-l10n-id="download_label"><?php esc_html_e( 'Download', 'pdf-embedder-premium' ); ?></span>
												</button>
											<?php endif; ?>

											<?php
											$fs_button_class = isset( $options['pdfemb_fullscreen'] ) && strtolower( $options['pdfemb_fullscreen'] ) === 'on' ? 'fsButtonOn' : 'fsButtonHidden';
											?>
											<button id="fullscreen-bottom" data-pdf-id="<?php echo esc_attr( $this->atts['pdfID'] ); ?>" class="toolbarButton wppdf-fullscreen-button wppdf-embed-fullscreen-<?php echo esc_attr( $this->atts['pdfID'] ); ?> <?php echo esc_attr( $fs_button_class ); ?>" title="<?php esc_attr_e( 'Fullscreen', 'pdf-embedder-premium' ); ?>" tabindex="35" data-l10n-id="fullscreen" data-popup="#pdf-data">
												<span data-l10n-id="fullscreen_label"><?php esc_html_e( 'Fullscreen', 'pdf-embedder-premium' ); ?></span>
											</button>
										</div>
										<div id="toolbarViewerMiddle-bottom" class="toolbarViewerMiddle">
											<div class="splitToolbarButton">
												<button id="zoomOut-bottom" class="zoomOut toolbarButton" title="<?php esc_attr_e( 'Zoom Out', 'pdf-embedder-premium' ); ?>" tabindex="21" data-l10n-id="zoom_out">
													<span data-l10n-id="zoom_out_label"><?php esc_html_e( 'Zoom Out', 'pdf-embedder-premium' ); ?></span>
												</button>
												<button id="zoomIn-bottom" class="zoomIn toolbarButton" title="<?php esc_attr_e( 'Zoom In', 'pdf-embedder-premium' ); ?>" tabindex="22" data-l10n-id="zoom_in">
													<span data-l10n-id="zoom_in_label"><?php esc_html_e( 'Zoom In', 'pdf-embedder-premium' ); ?></span>
												</button>
											</div>
											<span id="scaleSelectContainer" class="dropdownToolbarButton">
												<select id="scaleSelect-bottom" class="scaleSelect title="<?php esc_attr_e( 'Zoom', 'pdf-embedder-premium' ); ?>" tabindex="23" data-l10n-id="zoom">
													<option id="pageAutoOption-bottom" title="" value="auto" data-l10n-id="page_scale_auto"><?php esc_html_e( 'Automatic Zoom', 'pdf-embedder-premium' ); ?></option>
													<option id="pageActualOption-bottom" title="" value="page-actual" data-l10n-id="page_scale_actual"><?php esc_html_e( 'Actual Size', 'pdf-embedder-premium' ); ?></option>
													<option id="pageFitOption-bottom" title="" value="page-fit" selected="selected" data-l10n-id="page_scale_fit"><?php esc_html_e( 'Page Fit', 'pdf-embedder-premium' ); ?></option>
													<option id="pageWidthOption-bottom" title="" value="page-width" data-l10n-id="page_scale_width"><?php esc_html_e( 'Page Width', 'pdf-embedder-premium' ); ?></option>
													<option id="customScaleOption-bottom" title="" value="custom" disabled="disabled" hidden="true"></option>
													<option title="" value="0.5" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 50 }'>50%</option>
													<option title="" value="0.75" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 75 }'>75%</option>
													<option title="" value="1" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 100 }'>100%</option>
													<option title="" value="1.25" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 125 }'>125%</option>
													<option title="" value="1.5" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 150 }'>150%</option>
													<option title="" value="2" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 200 }'>200%</option>
													<option title="" value="3" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 300 }'>300%</option>
													<option title="" value="4" data-l10n-id="page_scale_percent" data-l10n-args='{ "scale": 400 }'>400%</option>

													<?php do_action( 'pdfemb_viewer_toolbar_scale_percent' ); ?>
												</select>
											</span>
										</div>
									</div>

									<?php
									// There should only be one loading bar for a viewer.
									if ( $toolbar === 'bottom' ) :
										?>
										<div id="loadingBar" class="loadingBar">
											<div class="progress">
												<div class="glimmer"></div>
											</div>
										</div>
									<?php endif; ?>
								</div>
							</div>
						<?php endif; ?>
					</div>

				</div>

				<?php
				/**
				 * Fires in the footer of the PDF viewer.
				 *
				 * @since 5.0.0
				 */
				do_action( 'wp_pdf_viewer_footer' );
				?>
			</body>
		</html>

		<?php
	}

	/**
	 * Render the PDF viewer inside the page content instead of the shortcode or block.
	 *
	 * @since 5.2.0
	 */
	public function render(): string { // phpcs:ignore Generic.Metrics.CyclomaticComplexity.TooHigh

		$GLOBALS['pdf_index'] = isset( $GLOBALS['pdf_index'] ) ? (int) $GLOBALS['pdf_index'] + 1 : 1;
		$index                = $GLOBALS['pdf_index'];
		$this->atts['index']  = $index;

		$width  = $this->atts['width'] ?? $this->options['pdfemb_width'];
		$width  = is_numeric( $width ) ? (int) $width . 'px' : '100%';
		$height = $this->atts['height'] ?? $this->options['pdfemb_height'];
		$height = is_numeric( $height ) ? (int) $height . 'px' : '1000px';

		// Get the attachment ID by its URL if not passed directly.
		$this->atts['pdfID'] = empty( $this->atts['pdfID'] ) ? PdfFile::get_id_by_url( $this->atts['url'] ?? '' ) : (int) $this->atts['pdfID'];

		$iframe_url = add_query_arg(
			/**
			 * Filters the attributes for the PDF viewer.
			 *
			 * @since 5.2.0
			 *
			 * @param array $atts The attributes for the PDF viewer.
			 */
			apply_filters( 'pdfemb_premium_viewer_render_atts', $this->atts ),
			home_url( '/' )
		);

		wp_enqueue_style(
			'pdf-fullscreen',
			Assets::url( 'css/pdfemb-fullscreen.css' ),
			null,
			Assets::ver()
		);

		ob_start();
		?>

		<div id="wppdfemb-frame-container-<?php echo esc_attr( $this->atts['pdfID'] ); ?>" style="-webkit-overflow-scrolling:auto;">
			<iframe src="<?php echo esc_url( $iframe_url ); ?>"
				id="wppdf-emb-iframe-<?php echo esc_attr( $this->atts['pdfID'] ); ?>"
				data-pdf-index="<?php echo esc_attr( $index ); ?>"
				class="pdfembed-iframe nonfullscreen"
				style="border:none;width:100%;max-width:<?php echo esc_attr( $width ); ?>;min-height:<?php echo esc_attr( $height ); ?>"
				scrolling="yes">
			</iframe>
		</div>

		<?php

		return ob_get_clean();
	}
}
